/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/

/* May, 27 2000 / Steven Behnke <sbehnke@digistatic.net>
 * Began the implementation of a hash table database address book.
 * Added all the database functions required.
 * Added a GUI for Add and Delete functions.
 *
 * May, 31 2000 / Steven Behnke <sbehnke@digistatic.net>
 * Added a GUI for the update function.
 *
 * June, 6 2000 / Steven Behnke <sbehnke@digistatic.net>
 * Created a new user interface for the addressbook.
 * Merged the addressbook into the existing MMC project.
 */

#ifdef HAVE_CONFIG_H
# include<config.h>
#endif
#include<gnome.h>
#include"main.h"
#include "address.h"
#include "compose.h"

GtkWidget *addressList;

GtkWidget *address_entry;
GtkWidget *name_entry;

gboolean addressAdd = false;
gboolean updateAddress = false;


class email_for_address {
public:
	gchar *reply_to;
	gchar *subject;
	gchar *quoted_text;

	email_for_address();
	~email_for_address();
};

email_for_address::email_for_address() {
	reply_to=NULL;
	subject=NULL;
	quoted_text=NULL;
}

email_for_address::~email_for_address() {
	if (reply_to) delete[] reply_to;
	if (subject) delete[] subject;
	if (quoted_text) g_free(quoted_text);

	reply_to=NULL;
	subject=NULL;
	quoted_text=NULL;
}

/*
 * Create the GUI Address book.
 */

GtkWidget *create_addressBook(void)
{
	// Widgets
	GtkWidget *addressBook;
	GtkWidget *vbox;
	GtkWidget *hbox1;
	GtkWidget *hbox2;
	GtkWidget *addButton;
	GtkWidget *deleteButton;
	GtkWidget *modifyButton;
	GtkWidget *scrolledWindow;
//	GtkWidget *addressList;
	GtkWidget *toButton;
	GtkWidget *ccButton;
	
	addressBook = gtk_window_new(GTK_WINDOW_TOPLEVEL);
//	gtk_widget_set_name(addressBook, "addressBook");
//	gtk_object_set_data(GTK_OBJECT(addressBook), "addressBook", addressBook);
	gtk_window_set_title(GTK_WINDOW(addressBook), _("MMC Address Book"));
	gtk_widget_set_usize(addressBook, 480, 320);
	
	vbox = gtk_vbox_new(false, 0);
	gtk_container_add(GTK_CONTAINER(addressBook), vbox);
	
	hbox1 = gtk_hbox_new(false, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox1, false, false, 0);
	
	// First row of buttons
	
	addButton = gtk_button_new_with_label(_("Add"));
	gtk_box_pack_start(GTK_BOX(hbox1), addButton, false, false, 0);
	gtk_widget_set_usize(addButton, 64, -2);
	gtk_container_set_border_width(GTK_CONTAINER(addButton), 5);
	
	deleteButton = gtk_button_new_with_label(_("Delete"));
	gtk_box_pack_start(GTK_BOX(hbox1), deleteButton, false, false, 0);
	gtk_widget_set_usize(deleteButton, 64, -2);
	gtk_container_set_border_width(GTK_CONTAINER(deleteButton), 5);
	
	modifyButton = gtk_button_new_with_label(_("Modify"));
	gtk_box_pack_start(GTK_BOX(hbox1), modifyButton, false, false, 0);
	gtk_widget_set_usize(modifyButton, 64, -2);
	gtk_container_set_border_width(GTK_CONTAINER(modifyButton), 5);
	
	// ScrolledWindow and CList
	
	scrolledWindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start(GTK_BOX(vbox), scrolledWindow, true, true, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledWindow),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	
	addressList = gtk_clist_new(2);
	gtk_widget_set_name(addressList, "addressList");
	gtk_container_add(GTK_CONTAINER(scrolledWindow), addressList);
	
	gtk_clist_set_column_title(GTK_CLIST(addressList), 0, _("Name"));
	gtk_clist_set_column_title(GTK_CLIST(addressList), 1, _("E-mail Address"));
	gtk_clist_column_titles_show(GTK_CLIST(addressList));
	
	// Add the records from the DB to the clist
	
	populateList(addressList);
	
	// Second row of buttons
	
	hbox2 = gtk_hbox_new(false, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox2, false, false, 0);
	
	toButton = gtk_button_new_with_label(_("To:"));
	gtk_box_pack_start(GTK_BOX(hbox2), toButton, true, true, 0);
	gtk_container_set_border_width(GTK_CONTAINER(toButton), 5);
	
	ccButton = gtk_button_new_with_label(_("CC:"));
	gtk_box_pack_start(GTK_BOX(hbox2), ccButton, true, true, 0);
	gtk_container_set_border_width(GTK_CONTAINER(ccButton), 5);
	
	// Set some callback functions for the buttons
	
	gtk_signal_connect(GTK_OBJECT(addButton), "clicked",
				GTK_SIGNAL_FUNC(entryDialog), addressList);
	gtk_signal_connect(GTK_OBJECT(deleteButton), "clicked",
				GTK_SIGNAL_FUNC(deleteAddress), addressList);
	gtk_signal_connect(GTK_OBJECT(modifyButton), "clicked",
				GTK_SIGNAL_FUNC(updateRecord), addressList);
	
	gtk_signal_connect(GTK_OBJECT(toButton), "clicked",
				GTK_SIGNAL_FUNC(composeFromAddy), addressList);
	
	gtk_widget_show_all(addressBook);
	
	return addressBook;
}

/* Open the database and lock the file to prevent others from modifying
 * it while we are reading from it.
 */

DB *openDatabase(int writeable)
{
	DB *addressDB;
	char *filename;
	int flags;
	struct flock lock;

	/* Set up the file lock */

	lock.l_whence = SEEK_SET;
	lock.l_start = 0;
	lock.l_len = 0;

	/* Set the open and lock modes */

	if (writeable) {
		flags = O_CREAT | O_RDWR;
		lock.l_type = F_WRLCK;
	}

	else {
		flags = O_RDONLY;
		lock.l_type = F_RDLCK;
	}

	filename=gnome_config_get_string("/mmc/advanced/addressfile=");
	if (filename==NULL || *filename==0) {
		filename = (char*)g_malloc(strlen(getenv("HOME")) + 20);
		strcpy(filename, g_mmcHome);
		strcat(filename, "addressbook.db");
	}

	addressDB = dbopen(filename, flags, 0666, DB_HASH, NULL);
	if (!addressDB) {
		g_warning(_("Error opening file '%s': %s"), filename, strerror(errno));
		return NULL;
	}

	g_free(filename);

	return addressDB;
}

/* Add an address record to the hash table using the e-mail address as the
 * key for the record.
 */

int dbAdd(gchar *email, gchar *name, gboolean update)
{
	DB *addressBookDB;
	DBT key, value;
	int rc = 0;
	
	if (!(addressBookDB = openDatabase(TRUE)))
		return 1;

	key.data = (char *)email;
	key.size = strlen((char *)email) + 1;

	if (!update) {
		rc = addressBookDB->get(addressBookDB, &key, &value, FALSE);
		if (rc == -1) {
			g_warning(_("Error: %s\n"), strerror(errno));
			rc = TRUE;
		}
		else if (!rc) {
		/*	char buf[200];
			snprintf(buf,200,_("%s already listed as %s\n"), (char *)email, (char *)value.data);
			gnome_warning_dialog(buf);
		*/
		//	TODO:  Make Error reply in a dialog box only if it is not set to update.
		//	From biesi: in theory, the above code should do it. it doesn't.

			rc = TRUE;
		}
		else
			rc = FALSE;

	}

	if (!rc) {
		value.data = (char *)name;
		value.size = strlen((char *)name) + 1;

		if (addressBookDB->put(addressBookDB, &key, &value, false)) {
			g_warning(_("Insert failed: %s"), strerror(errno));
			rc = 1;
		}
	}

	addressBookDB->close(addressBookDB);

	return rc;
}

/* Delete a record when provided with an e-mail address.
 * Properly handle errors when the address is not listed.
 */

int dbDelete(gchar *email)
{
	DB *addressBookDB;
	DBT key;
	int rc;

	if (!(addressBookDB = openDatabase(true)))
		return 1;
	
	key.data = (char *)email;
	key.size = strlen((char *)email) + 1;
	
	rc = addressBookDB->del(addressBookDB, &key, false);
	if (rc == -1) {
		gchar buf[200];
		snprintf(buf,200,_("Delete failed: %s\n"), strerror(errno));
		gnome_warning_dialog(buf);
		rc = 1;
	}
	else if (rc) {
		g_warning(_("%s is not listed\n"), (char *)email);
		rc = 1;
	}

	addressBookDB->close(addressBookDB);

	return rc;
}

/*
 * More testing functions here...
 */
/*
gint Destroy(GtkWidget *widget, gpointer *data)
{
	gtk_main_quit();
	return FALSE;
}
*/

/* Read all of the records from the database and put them into the
 * clist so that we can actually view them.
 */

int populateList(GtkWidget *clist)
{
	DB *addressBookDB;
	DBT key, value;
	int rc;
	int flags = R_FIRST;
	gchar *szData[2];
	
	gtk_clist_freeze(GTK_CLIST(clist));
	
	if (!(addressBookDB = openDatabase(false)))
		return 1;

	while (!(rc = addressBookDB->seq(addressBookDB, &key, &value, flags))) {
		flags = R_NEXT;
		// add to clist...
		
		szData[0] = (char *)value.data;
		szData[1] = (char *)key.data;

		gtk_clist_append(GTK_CLIST(clist), szData);
	}

	if (rc == -1) {
		g_warning(_("Seq access failed: %s"), strerror(errno));
		rc = 1;
	}
	else {
		rc = 0;
	}
	
	gtk_clist_columns_autosize(GTK_CLIST(clist));
	
//	Doesn't seem to work correctly.
//	gtk_clist_set_column_width(GTK_CLIST(clist), 0, 200);

	gtk_clist_thaw(GTK_CLIST(clist));
	
	addressBookDB->close(addressBookDB);

	return rc;
}

/*
 * Close Dialog callback function
 */

void CloseDialog(GtkWidget *widget, gpointer *data)
{
	gchar *szData[2];	
	
	if (addressAdd) {
		szData[1] = gtk_entry_get_text(GTK_ENTRY(name_entry));
		szData[0] = gtk_entry_get_text(GTK_ENTRY(address_entry));
		
		if (!strcmp(szData[1], "\0") || !strcmp(szData[0], "\0"))
			return;	
		
		if (!dbAdd(szData[0], szData[1], FALSE)) {
			gtk_clist_append(GTK_CLIST(addressList), szData);
			gtk_clist_columns_autosize(GTK_CLIST(addressList));
		}
	}
	
	if (updateAddress) {
		char *email1;
		char *name1;
		char *email2;
		char *name2;
		
		if (GTK_CLIST(addressList)->selection) {
			int row = GPOINTER_TO_INT(GTK_CLIST(addressList)->selection->data);
			gtk_clist_get_text(GTK_CLIST(addressList), row, 0, &name1);
			gtk_clist_get_text(GTK_CLIST(addressList), row, 1, &email1);
			name2 = gtk_entry_get_text(GTK_ENTRY(name_entry));
			email2 = gtk_entry_get_text(GTK_ENTRY(address_entry));
			
			if (!strcmp(name1, "\0") || !strcmp(email1, "\0"))
				return;
			
			if (!strcmp(email1, email2)) {
				dbAdd(email2, name2, true);
				gtk_clist_set_text(GTK_CLIST(addressList), row, 0, name2);
				updateAddress = false;
			}
			
			else {
				dbDelete(email1);
				dbAdd(email2, name2, false);
				gtk_clist_set_text(GTK_CLIST(addressList), row, 0, name2);
				gtk_clist_set_text(GTK_CLIST(addressList), row, 1, email2);
				updateAddress = false;
			}
		}
	}
	gtk_widget_destroy(GTK_WIDGET(data));
}

/*
 * Closing Dialog call back function
 */

void ClosingDialog(GtkWidget *widget, gpointer *data)
{
	addressAdd = TRUE;
	gchar *szData[2];	

	if (addressAdd) {
		szData[0] = gtk_entry_get_text(GTK_ENTRY(name_entry));
		szData[1] = gtk_entry_get_text(GTK_ENTRY(address_entry));
		
		if (!strcmp(szData[1], "\0") || !strcmp(szData[0], "\0"))
			return;

		if (!dbAdd(szData[1], szData[0], FALSE)) {
			gtk_clist_append(GTK_CLIST(addressList), szData);
			gtk_clist_columns_autosize(GTK_CLIST(addressList));
		}
	}

	gtk_grab_remove(GTK_WIDGET(widget));
}

/*
 * Dialog box to accept the user input.
 */

int entryDialog(GtkWidget *widget, gpointer *data)
{
	GtkWidget *button;
	GtkWidget *label;
	GtkWidget *dialog_window;

	addressAdd = FALSE;

	dialog_window = gtk_dialog_new();

	gtk_signal_connect(GTK_OBJECT(dialog_window), "destroy",
				GTK_SIGNAL_FUNC(ClosingDialog), dialog_window);

	gtk_window_set_title(GTK_WINDOW(dialog_window), _("New E-mail Entry"));

	label = gtk_label_new(_("Name"));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->vbox), label,
			TRUE, TRUE, 0);

	name_entry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->vbox), name_entry,
			TRUE, TRUE, 0);

	label = gtk_label_new(_("E-mail address"));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->vbox), label,
			TRUE, TRUE, 0);

	address_entry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->vbox), address_entry,
			TRUE, TRUE, 0);

	button = gtk_button_new_with_label(_("OK"));
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
				GTK_SIGNAL_FUNC(CloseDialog), dialog_window);

	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog_window)->vbox), button,
			TRUE, TRUE, 0);
	
	if (updateAddress) {
		char *email;
		char *name;
		if (GTK_CLIST(addressList)->selection) {
			int row = GPOINTER_TO_INT(GTK_CLIST(addressList)->selection->data);
			gtk_clist_get_text(GTK_CLIST(addressList), row, 0, &name);
			gtk_clist_get_text(GTK_CLIST(addressList), row, 1, &email);
			gtk_entry_set_text(GTK_ENTRY(name_entry), name);
			gtk_entry_set_text(GTK_ENTRY(address_entry), email);
		}
	}

	gtk_widget_grab_default(button);
	gtk_widget_show_all(dialog_window);
	gtk_grab_add(dialog_window);

	return 0;
}

/*
 * Delete Address callback function
 */

void deleteAddress(GtkWidget *widget, gpointer *data)
{
	char *email;
	int row;
	
	if (GTK_CLIST(addressList)->selection) {
		row = GPOINTER_TO_INT(GTK_CLIST(addressList)->selection->data);
		gtk_clist_get_text(GTK_CLIST(addressList), row, 1, &email);
//		fprintf(stderr, "Delete %s?\n", email);
		dbDelete(email);
		gtk_clist_remove(GTK_CLIST(addressList), row);
	}
	else
		gnome_warning_dialog(_("Nothing selected.\n"));
}

/*
 * Switch the update flag on.
 */

void updateRecord(GtkWidget *widget, gpointer *data)
{
	updateAddress = true;
	entryDialog(widget, data);
}

void open_book(GtkWidget *widget, gpointer *data) {
	/* GtkWidget *AddyBook = */ create_addressBook();
}

void composeFromAddy(GtkWidget *widget, gpointer *data) {
	char *email;
	char *name;
	GString *gstring;
	
	if (GTK_CLIST(addressList)->selection) {
		int row = GPOINTER_TO_INT(GTK_CLIST(addressList)->selection->data);
		gtk_clist_get_text(GTK_CLIST(addressList), row, 0, &name);
		gtk_clist_get_text(GTK_CLIST(addressList), row, 1, &email);
		
		email_for_address *mail=new email_for_address;

		gstring = g_string_new("\0");
		g_string_append(gstring, name);
		g_string_append(gstring, " <");
		g_string_append(gstring, email);
		g_string_append(gstring, ">");
		
		fprintf(stderr, "%s\n", (char *)gstring->str);
		
		mail->reply_to = (char *)gstring->str;
//		mail->subject = "I'm a smuck";
//		mail->quoted_text = "So am I";
		
		compose_msg(NULL, mail);
	}
}
