/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/

/*  April, 01 2000 / Steven Behnke <sbehnke@digistatic.net>
 *  
 *	Added a new user interface for the compose window.
 *	Lots of new widgets without removing any functionality.
 */

#ifdef HAVE_CONFIG_H
#  include<config.h>
#endif

#include<gnome.h>
#include<locale.h>
#include<string.h>
#include<stdio.h>
#include<sys/types.h>
#include<sys/stat.h>
#include"main.h"
#include"compose.h"
#include"smtp.h"
#include"defaults.h"
#include"msg.h"

#if 0
class email_for_reply {
public:
	gchar *reply_to;
	gchar *subject;
	gchar *quoted_text;

	email_for_reply();
	~email_for_reply();
};

email_for_reply::email_for_reply() {
	reply_to=NULL;
	subject=NULL;
	quoted_text=NULL;
}

email_for_reply::~email_for_reply() {
	if (reply_to) delete[] reply_to;
	if (subject) delete[] subject;
	if (quoted_text) delete[] quoted_text;

	reply_to=NULL;
	subject=NULL;
	quoted_text=NULL;
}
#endif

static GnomeUIInfo compose_toolbar[] = {
	GNOMEUIINFO_ITEM_STOCK(N_("Send"), N_("Sends the message now"), sendnow, GNOME_STOCK_PIXMAP_MAIL_SND),
	GNOMEUIINFO_ITEM_STOCK(N_("Cancel"), N_("Cancel"), destroy0, GNOME_STOCK_PIXMAP_CLOSE),
	GNOMEUIINFO_END
};

static GnomeUIInfo compose_menu_message[] = {
	GNOMEUIINFO_ITEM_STOCK(N_("Send now"), N_("Sends the message now"), sendnow, GNOME_STOCK_PIXMAP_MAIL_SND),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK(N_("Close"), N_("Cancel"), destroy0, GNOME_STOCK_PIXMAP_CLOSE),
	GNOMEUIINFO_END
};

static GnomeUIInfo compose_menus[] = {
	GNOMEUIINFO_SUBTREE_HINT(N_("Message"), N_("Commands for the Message"), compose_menu_message),
	GNOMEUIINFO_END
};

void CMessage::reply() {
	fill();
	gchar *quoted = quote_text(body);
	gchar *newSubj = new gchar[strlen(subj)+5];
	if (	(strncasecmp(subj,"Re:",3) == 0) ||
		(strncasecmp(subj,"AW:",3) == 0)) {
		strcpy(newSubj,subj);
	}
	else {
		sprintf(newSubj,"Re: %s",subj);
	}
	gchar *useRep;
	if (replyto) useRep = replyto;
	else useRep = from;

	CMessage newMsg(NULL, NULL, useRep, cc, bcc, newSubj, NULL, quoted);
	newMsg.compose();

	delete[] quoted;
	delete[] newSubj;
}

void CMessage::compose() {
	GtkWidget *compose_wnd;
	GtkWidget *mail_field;
//	GtkWidget *buttonbox;
	GtkWidget *fixedLayout;
	GtkWidget *hseparator;
	GtkWidget *vbox;
	GtkWidget *msgStatus;
	GtkWidget *scrolledwindow;

	GtkWidget *efrom;
	GtkWidget *eto;
	GtkWidget *esubject;

	GtkWidget *ecc, *ebcc;
	
	GtkWidget *fromLabel;
	GtkWidget *toLabel;
	GtkWidget *subjectLabel;

	GtkWidget *ccLabel, *bccLabel;
#if 0
	GtkWidget *sndButton;
	GtkWidget *cancelButton;
#endif
	GdkFont *mail_font;

	GtkWidget **senddata;

	gchar *fontName = gnome_config_get_string("/mmc/fonts/mail=" DEFAULT_MAIL_FONT);
	mail_font=gdk_font_load(fontName);
	g_free(fontName);
	
	//compose_wnd = gtk_window_new(GTK_WINDOW_DIALOG);
	compose_wnd = gnome_app_new(PACKAGE "_compose", _("Compose a Message"));
	//gtk_window_set_title(GTK_WINDOW(compose_wnd), _("New E-mail"));
	

	// Layout pieces

	vbox = gtk_vbox_new(FALSE, 0);
	//gtk_container_add(GTK_CONTAINER(compose_wnd), vbox);
	gnome_app_set_contents(GNOME_APP(compose_wnd), vbox);
	
	fixedLayout = gtk_fixed_new();
	gtk_box_pack_start(GTK_BOX(vbox), fixedLayout, FALSE, FALSE, 0);

	// Labels

	fromLabel = gtk_label_new(_("From:"));
	gtk_fixed_put (GTK_FIXED(fixedLayout), fromLabel, 8, 8);
	gtk_widget_set_uposition(fromLabel, 8, 8);
	gtk_widget_set_usize(fromLabel, 64, 24);
	gtk_label_set_justify(GTK_LABEL(fromLabel), GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap(GTK_LABEL(fromLabel), TRUE);

	toLabel = gtk_label_new(_("To:"));
	gtk_fixed_put(GTK_FIXED(fixedLayout), toLabel, 8, 32);
	gtk_widget_set_uposition(toLabel, 8, 32);
	gtk_widget_set_usize(toLabel, 64, 24);
	gtk_label_set_justify(GTK_LABEL(toLabel), GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap(GTK_LABEL(toLabel), TRUE);

	subjectLabel = gtk_label_new(_("Subject:"));
	gtk_fixed_put(GTK_FIXED(fixedLayout), subjectLabel, 8, 56);
	gtk_widget_set_uposition(subjectLabel, 8, 56);
	gtk_widget_set_usize(subjectLabel, 64, 24);
	gtk_label_set_justify(GTK_LABEL(subjectLabel), GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap(GTK_LABEL(subjectLabel), TRUE);

	ccLabel = gtk_label_new(_("CC:"));
	gtk_fixed_put(GTK_FIXED(fixedLayout), ccLabel, 8, 80);
	gtk_widget_set_uposition(ccLabel, 8, 80);
	gtk_widget_set_usize(ccLabel, 64, 24);
	gtk_label_set_justify(GTK_LABEL(ccLabel), GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap(GTK_LABEL(ccLabel), TRUE);

	bccLabel = gtk_label_new(_("BCC:"));
	gtk_fixed_put(GTK_FIXED(fixedLayout), bccLabel, 8, 104);
	gtk_widget_set_uposition(bccLabel, 8, 104);
	gtk_widget_set_usize(bccLabel, 64, 24);
	gtk_label_set_justify(GTK_LABEL(bccLabel), GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap(GTK_LABEL(bccLabel), TRUE);

						

	// Text Entries

	efrom = gnome_entry_new("from");
	GtkWidget *gfrom=gnome_entry_gtk_entry(GNOME_ENTRY(efrom));
	gtk_entry_set_text(GTK_ENTRY(gfrom), gnome_config_get_string("/mmc/smtp/from="));
	gtk_fixed_put(GTK_FIXED(fixedLayout), efrom, 80, 8);
	gtk_widget_set_uposition (efrom, 80, 8);
	gtk_widget_set_usize (efrom, 344, 24);

	eto = gnome_entry_new("to");
	gtk_fixed_put(GTK_FIXED(fixedLayout), eto, 80, 32);
	gtk_widget_set_uposition(eto, 80, 32);
	gtk_widget_set_usize(eto, 344, 24);

	esubject = gtk_entry_new();
	gtk_fixed_put (GTK_FIXED (fixedLayout), esubject, 80, 56);
	gtk_widget_set_uposition (esubject, 80, 56);
	gtk_widget_set_usize (esubject, 344, 24);

	// CC and BCC
	ecc = gnome_entry_new("cc");
	gtk_fixed_put (GTK_FIXED (fixedLayout), ecc, 80, 80);
	gtk_widget_set_uposition (ecc, 80, 80);
	gtk_widget_set_usize (ecc, 344, 24);

	ebcc = gnome_entry_new("bcc");
        gtk_fixed_put (GTK_FIXED (fixedLayout), ebcc, 80, 104);
        gtk_widget_set_uposition (ebcc, 80, 104);
        gtk_widget_set_usize (ebcc, 344, 24);
	
	// Hseparator

	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), hseparator, FALSE, FALSE, 0);

	// Scrolled text box...

	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start (GTK_BOX (vbox), scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

	mail_field = gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT(mail_field),TRUE);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), mail_field);

	// Status bar, just in case...
	// Important information can be pushed on the the status bar.

        msgStatus=gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_USER);
	gnome_appbar_set_default(GNOME_APPBAR(msgStatus), _("Composing Message"));

	// Senddata widget array
	
	senddata = new GtkWidget*[8];
	senddata[0]=compose_wnd;
	senddata[1]=eto;
	senddata[2]=esubject;
	senddata[3]=mail_field;
	senddata[4]=efrom;

	senddata[5]=ecc;
	senddata[6]=ebcc;
	senddata[7]=msgStatus;

	gnome_app_create_toolbar_with_data(GNOME_APP(compose_wnd), compose_toolbar, senddata);
	gnome_app_create_menus_with_data(GNOME_APP(compose_wnd), compose_menus, senddata);
	gnome_app_set_statusbar(GNOME_APP(compose_wnd), msgStatus);

	gnome_app_install_menu_hints(GNOME_APP(compose_wnd), compose_menus);

	gtk_text_set_point(GTK_TEXT(mail_field),0);
	gtk_text_forward_delete(GTK_TEXT(mail_field),gtk_text_get_length(GTK_TEXT(mail_field)));
					

	// If replying, fill out To, Subject & Text
	if (body) {
			gtk_text_insert(GTK_TEXT(mail_field), mail_font, NULL, NULL,
			body, -1);

	}
	else // to set the font:
		gtk_text_insert(GTK_TEXT(mail_field), mail_font, NULL, NULL,
			"\n", 1);

	if (from) {
		GtkWidget *gfrom = gnome_entry_gtk_entry(GNOME_ENTRY(efrom));
		gtk_entry_set_text(GTK_ENTRY(gfrom), from);
	
	}
	if (to) {
		GtkWidget *gto;
		gto=gnome_entry_gtk_entry(GNOME_ENTRY(eto));
		gtk_entry_set_text(GTK_ENTRY(gto), to);
	}
	if (cc) {
		GtkWidget *gcc; // no, not the compiler ;)
		gcc = gnome_entry_gtk_entry(GNOME_ENTRY(ecc));
		gtk_entry_set_text(GTK_ENTRY(gcc), cc);
	}
	if (bcc) {
		GtkWidget *gbcc = gnome_entry_gtk_entry(GNOME_ENTRY(ebcc));
		gtk_entry_set_text(GTK_ENTRY(gbcc), bcc);
	}
	if (subj)
		gtk_entry_set_text(GTK_ENTRY(esubject), subj);


	gtk_text_set_point(GTK_TEXT(mail_field), 0);

	gtk_widget_set_usize(compose_wnd,500,500);
	gtk_widget_show_all(compose_wnd);

}

#if 0
// don't want to have everything in memory
gchar *CMessage::getBody() {
	if (!body) return NULL;
	gchar *ret = new gchar[strlen(body)+1];
	strcpy(ret,body);
	return ret;
}

gchar *CMessage::getHead(gboolean all) {
	GString *ret;
	ret = g_string_new("");
	if (all && ((additional->len) > 0)) g_string_append(ret,
		additional->str);
	if (from) g_string_append(ret, from);
	if (to) g_string_append(ret, to);
	if (cc) g_string_append(ret, cc);
	if (bcc) g_string_append(ret, bcc);
	if (date) g_string_append(ret, date);
	gchar *pcRet = new gchar[ret->len+1];
	strcpy(pcRet, ret->str);
	g_string_free(ret);
	return pcRet;
}
#endif

gchar *CMessage::getRaw() {
	return folder->get(data);
}

// char *quote_text(char *text);
// Quotes the Text, e.g. > this is what you wrote
// Returned String must be delete[]ed later.
gchar * quote_text(gchar *text) {
	if (!text || !*text) return text;
	guint len=strlen(text);
	gchar *out=new gchar[len*3];

	gchar *ptr=out;

	memcpy(out,"> ",2);
	out+=2;
	while (*text) {
		*out=*text;
		if (*text=='\n') {
			out[1]='>';
			out[2]=' ';
			out+=2;
		}

		out++; text++;
	}
	*out=0;
	return ptr;
}

void destroy(GtkWidget *foo, gpointer data) {

	GtkWidget *widget = (GtkWidget*)data;
	gtk_widget_destroy(widget);
}

void destroy0(GtkWidget *foo, gpointer data) {
	g_assert(data != NULL);
	GtkWidget **widget = (GtkWidget**)data;
	g_assert(*widget != NULL);
	destroy(foo,*widget);
	*widget=NULL;
}


void compose_msg(GtkWidget *widget, gpointer data) {
	CMessage* newMsg = new CMessage;
	newMsg->compose();
}


// Returns the number of recipients
int parse_rcpts(gchar* rcpts[], const gchar *to) {
	// delimiter is ,

	gchar *ptr;
	gchar *address;
	guint len, count=0;
	for (;;) {
		ptr=strstr(to,",");
		if (ptr) len=ptr-to;
		else len=strlen(to);

		rcpts[count]=new gchar[len];
		memcpy(rcpts[count], to, len);
		rcpts[count][len]=0;

		address=rcpts[count];
		ptr=strstr(address,"<");
		if (ptr) {
			address=g_strdup(ptr+1);
			delete[] rcpts[count];
			rcpts[count]=address;
			ptr=strstr(address,">");
			if (ptr) *ptr=0;
		}
		

		count++;		
		to=strstr(to,",");
		if (!to) break;
		to++;
		while (*to==' ') to++;

	}
	rcpts[count]=NULL;

	return count;
}

void sendnow(GtkWidget *widget, gpointer data) {
	GtkWidget **widgets=(GtkWidget**)data;

	GtkWidget *gfrom=gnome_entry_gtk_entry(GNOME_ENTRY(widgets[4]));
	GtkWidget *gto=gnome_entry_gtk_entry(GNOME_ENTRY(widgets[1]));

	GtkWidget *gcc=gnome_entry_gtk_entry(GNOME_ENTRY(widgets[5]));
	GtkWidget *gbcc=gnome_entry_gtk_entry(GNOME_ENTRY(widgets[6]));

	gchar *from = gtk_entry_get_text(GTK_ENTRY(gfrom));
	gchar *froms[10]; // temporary variable holding an array of from addresses.
	                  // will probably never contain more than one.
	gchar *to   = gtk_entry_get_text(GTK_ENTRY(gto));
	gchar *subject = gtk_entry_get_text(GTK_ENTRY(widgets[2]));
	gchar *body = gtk_editable_get_chars(GTK_EDITABLE(widgets[3]),0,-1);

	gchar *cc = gtk_entry_get_text(GTK_ENTRY(gcc));
	gchar *bcc = gtk_entry_get_text(GTK_ENTRY(gbcc));

	if (!from || !*from) {
		gnome_app_error(GNOME_APP(widgets[0]), _("Please enter a from "
			"address!"));
		return;
	}
	if (!to || !*to) {
		gnome_app_error(GNOME_APP(widgets[0]), _("Please enter a "
			"recipient!"));
		return;
	}

	gnome_config_set_string("/mmc/smtp/from",from);
	gnome_config_sync();


	parse_rcpts(froms,from);
	g_free(from);
	from = froms[0];
	
	gchar *rcpts[100];
	int rCount; // the number of recipients
	rCount = parse_rcpts(rcpts,to);

	char today[32];
	time_t now=time(NULL);

	char *old_locale = setlocale(LC_TIME, NULL);
	setlocale(LC_TIME,"C");
	strftime(today,100,"%a, %d %b %Y %H:%M:%S %Z", localtime(&now));
	setlocale(LC_TIME, old_locale);

	gchar *add_head="MIME-Version: 1.0\nContent-Type: text/plain\n"
			"Content-Transfer-Encoding: 8bit\n"
			"X-Mailer: " PACKAGE " " VERSION "\n";
	guint length=strlen(body)+strlen(subject)+25+
		strlen(add_head)+strlen(from)+strlen(to)+
		strlen(cc) + strlen(bcc) + strlen(today);
       	// 25=Subject: + From: + reserve

	gchar *mdata=new gchar[length];
	sprintf(mdata,"%sFrom: %s\nSubject: %s\nDate: %s\n", add_head, from,
		subject, today);
	if (cc && *cc) {
		sprintf(mdata,"%sCC: %s\n",mdata,cc);
		rCount += parse_rcpts(rcpts+rCount, cc);
	}
	if (bcc && *bcc) {
		gchar *bccdata = new gchar[ strlen(mdata) + strlen(bcc) + strlen(body) + 15];
		char **bccRcpts = NULL;
		sprintf(bccdata, "%sBCC: %s\n\n%s\n\n", mdata, bcc, body);
		parse_rcpts(bccRcpts, bcc);
		send_mail(NULL, from, bccRcpts, bccdata);
		delete[] bccdata;
		for (gint i=0; bccRcpts[i]; i++) delete[] bccRcpts[i];
		
	}
	sprintf(mdata,"%s\n%s\n\n",mdata,body);
	
	if (send_mail(widgets[0], from,  rcpts, mdata) == 0) {
		CFolder *sentFolder=g_folders.get("Sent"); // SENT
		if (sentFolder) sentFolder->addMail(mdata);
		else gnome_app_warning(GNOME_APP(app),_("Could not write to FCC file!"));
	}
	for (int i=0; rcpts[i]; i++) delete[] rcpts[i];
	g_free(body);
	delete[] mdata;

}


void reply_to(GtkWidget *widget, gpointer data) {

	CMessage *msg = (CMessage*)gtk_clist_get_row_data(GTK_CLIST(mail_list), get_row(GTK_CLIST(mail_list)));
	msg->reply();

#if 0
	email_for_reply *mail=new email_for_reply;
	
	CMessage *msg = (CMessage*)gtk_clist_get_row_data(GTK_CLIST(mail_list), get_row(GTK_CLIST(mail_list)));

	gchar *mailtext=gtk_editable_get_chars(GTK_EDITABLE(par->text),0,-1);

	gboolean noFrom=FALSE;
	do {
		if (*mailtext=='\n' && *(mailtext+1)=='\n') {
			mailtext++;
			mail->quoted_text=quote_text(mailtext);
			break;
		}
		// We are in the header.
		if (memcmp(mailtext,"From:",5)==0 && !noFrom ) {
			noFrom=TRUE;
			gchar *end=strstr(mailtext,"\n");
			guint len=end-mailtext-6;
			
			mail->reply_to=new gchar[len];
			memcpy(mail->reply_to, mailtext+6, len);
			mail->reply_to[len]=0;
		}
		else if (memcmp(mailtext,"Reply-To:",9)==0) {
			noFrom=TRUE;
			if (mail->reply_to) delete[] mail->reply_to;
			gchar *end=strstr(mailtext,"\n");
			guint len=end-mailtext-9;
			
			mail->reply_to=new gchar[len];
			memcpy(mail->reply_to, mailtext+10, len);
			mail->reply_to[len]=0;
		}
		else if (memcmp(mailtext,"Subject:",8)==0) {
			gchar *end=strstr(mailtext,"\n");
			guint len=end-mailtext-8;
																
			mail->subject=new gchar[len+3];

			if (memcmp(mailtext,"Re:",3)!=0) {
				memcpy(mail->subject,"Re: ",3);
				memcpy(mail->subject+3, mailtext+8, len);
				mail->subject[len+3]=0;
			}
			else {
				memcpy(mail->subject, mailtext+8, len);
				mail->subject[len]=0;
			}
		}
	} while (*++mailtext);

	compose_msg(NULL, mail);
#endif
}


