/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/

#ifdef HAVE_CONFIG_H
# include<config.h>
#endif
#include <gnome.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include "main.h"
#include "compose.h"
#include "msg.h"
#include "folders.h"
#include "defaults.h"

void mboxCreate( gchar *string, gpointer data) {
	gchar *fname;
	if (!string || !*string) return;
	fname = new gchar[strlen(string) + 5];
	sprintf(fname,"{MD}%s",string);

	gchar *real_fn=getName(fname);
	
	struct stat folder_stats;
	if (!((stat(real_fn, &folder_stats)==-1) && (errno = ENOENT)))
		gnome_app_error(GNOME_APP(app), _("File already exists!"));
	
	FILE* folder;
	folder=fopen(real_fn,"a");
	fclose(folder);
	g_free(real_fn);

	((CFolderFile*)data)->doAdd(string,fname);
	
	g_free(string);	
	delete[] fname;
}


int CFolderFile::create() {
	gnome_app_request_string(GNOME_APP(app),
		_("Please enter the Name of the new folder"),
		mboxCreate, this);
	return 0;
}

int CFolderFile::addFolder() {
	GtkWidget *dialog;
	GtkWidget *box, *fixed;
	GtkWidget *wFilename, *wDisplayname;
	GtkWidget *lblFilename, *lblDisplayname;

	dialog=gnome_dialog_new(_("Add a folder"),
		GNOME_STOCK_BUTTON_OK,
		GNOME_STOCK_BUTTON_CANCEL,
		NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	box=GNOME_DIALOG(dialog)->vbox;

	fixed=gtk_fixed_new();

	wFilename=gnome_file_entry_new("folder_file", _("Choose the folder to add"));
	wDisplayname=gtk_entry_new();

	gtk_box_pack_start(GTK_BOX(box), fixed, TRUE, TRUE, 0);

	// Labels
	lblFilename = gtk_label_new(_("Filename:"));
	lblDisplayname = gtk_label_new(_("Name:"));

	gtk_fixed_put(GTK_FIXED(fixed), lblFilename, 8, 8);
	gtk_fixed_put(GTK_FIXED(fixed), lblDisplayname, 8, 32);
		
	// Add the entries
	gtk_fixed_put(GTK_FIXED(fixed), wFilename, 80, 8);
	gtk_fixed_put(GTK_FIXED(fixed), wDisplayname, 80, 32);

	gtk_widget_show_all(box);
	if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0) {
		gchar *filename, *displayname;
		filename = gtk_entry_get_text( GTK_ENTRY( gnome_file_entry_gtk_entry(
			GNOME_FILE_ENTRY(wFilename))));
		displayname = gtk_entry_get_text( GTK_ENTRY(wDisplayname));
		doAdd(displayname, filename);
	}
	gnome_dialog_close(GNOME_DIALOG(dialog));

	return 0;
}


void CFolderFile::doAdd(gchar *string, gchar *fname) {
	if (!string || !fname || !*string || !*fname) return;
	gchar *folder_string=gnome_config_get_string("/mmc/General/MailboxesFilenames=" DEFAULT_FOLDERS_FILE);
	gchar *new_folders = new gchar[ strlen(folder_string) + strlen(fname) + 2];

	sprintf(new_folders,"%s:%s",folder_string,fname);

	gnome_config_set_string("/mmc/General/MailboxesFilenames",new_folders);
	g_free(folder_string);
	g_free(new_folders);

	folder_string=gnome_config_get_string("/mmc/General/MailboxesTitles=" DEFAULT_FOLDERS_TITLE);
	new_folders = new gchar[ strlen(folder_string) + strlen(string) + 2];

	sprintf(new_folders,"%s:%s",folder_string,string);

	gnome_config_set_string("/mmc/General/MailboxesTitles", new_folders);
	g_free(folder_string);
	g_free(new_folders);

	gnome_config_sync();
						

	gtk_clist_append(GTK_CLIST(mboxes), &string);

	CFolderFile *newFolder = new CFolderFile(getName(fname), fname, string, GTK_CLIST(::mail_list), GTK_TEXT(view_mail));
	gtk_clist_set_row_data(GTK_CLIST(mboxes), (GTK_CLIST(mboxes)->rows)-1, newFolder);

	g_folders.add(newFolder, string);

}

void doAdd(GtkWidget *widget, gpointer data) {
	gint row = get_row(GTK_CLIST(data));
	GtkWidget *w = gtk_widget_get_ancestor(GTK_WIDGET(data),
		gtk_type_from_name("GnomeDialog"));
	GtkWidget *list = (GtkWidget*)data;
	if (row < 0) {
		gnome_error_dialog_parented(_("Please choose a folder type from the List!"),
			GTK_WINDOW(w));
		return;
	}
	gchar *folderType;
	gtk_clist_get_text(GTK_CLIST(list), row, 0, &folderType);
	CFolder *folder = g_fHandler.get(folderType);
	g_free(folderType);
	
	folder->addFolder();
	
}


void doCreate(GtkWidget *widget, gpointer data) {
	gint row = get_row(GTK_CLIST(data));
	GtkWidget *w = gtk_widget_get_ancestor(GTK_WIDGET(data),
		gtk_type_from_name("GnomeDialog"));
	GtkWidget *list = (GtkWidget*)data;
	if (row < 0) {
		gnome_error_dialog_parented(_("Please choose a folder type from the List!"),
			GTK_WINDOW(w));
		return;
	}
	gchar *folderType;
	gtk_clist_get_text(GTK_CLIST(list), row, 0, &folderType);
	CFolder *folder = g_fHandler.get(folderType);
	g_free(folderType);
	
	folder->create();
	
}

int CFolderFile::remove(gboolean onlyfromlist) {
	// gchar* desc;
	gchar *descs = gnome_config_get_string("/mmc/General/MailboxesTitles=" DEFAULT_FOLDERS_TITLE);
	gchar *fnames = gnome_config_get_string("/mmc/General/MailboxesFilenames=" DEFAULT_FOLDERS_FILE);

	gchar *newDesc = new gchar[strlen(descs) + 1];
	gchar *newFnames = new gchar[strlen(fnames) + 1];

	gchar *x = strstr(descs, description);
	if (x) {
		*(x-1) = 0;
		x++;
		while (*x != 0 && *x != ':') x++;
	}
	sprintf(newDesc, "%s%s", descs, x);
	g_free(descs);

	x = strstr(fnames, saved_name);
	if (x) {
		*(x-1) = 0;
		x++;
		while (*x != 0 && *x != ':') x++;
	}
	sprintf(newFnames, "%s%s", fnames, x);
	g_free(fnames);

	gnome_config_set_string("/mmc/General/MailboxesTitles", newDesc);
	gnome_config_set_string("/mmc/General/MailboxesFilenames", newFnames);
	gnome_config_sync();

	g_folders.remove(description);

	for (gint i=0; i < GTK_CLIST(mboxes)->rows; i++) {
		gchar *txt;
		gtk_clist_get_text(GTK_CLIST(mboxes), i, 0, &txt);
		if (strcmp(description, txt) == 0) {
			gtk_clist_remove(GTK_CLIST(mboxes), i);
		}
	}

	if (!onlyfromlist) {
		unlink(filename);
	}
	return 0;
}

void new_folder(GtkWidget *widget, gpointer data) {
	GtkWidget *chooser;
	GtkWidget *box;
	GtkWidget *scr_wnd;
	GtkWidget *list;
	
	chooser = gnome_dialog_new(_("Choose Folder Type"),
		GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);

	gnome_dialog_set_parent(GNOME_DIALOG(chooser), GTK_WINDOW(app));
		
	box = GNOME_DIALOG(chooser)->vbox;
	
	list = g_fHandler.makeCList(FLAG_SEND|FLAG_RECV);
	
	scr_wnd = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scr_wnd),
		GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(scr_wnd), list);

	gtk_box_pack_start(GTK_BOX(box), scr_wnd, TRUE, TRUE, 0);

	gnome_dialog_button_connect(GNOME_DIALOG(chooser), 0, 
		GTK_SIGNAL_FUNC(doCreate), list);
	gnome_dialog_button_connect(GNOME_DIALOG(chooser), 0, 
		GTK_SIGNAL_FUNC(destroy), chooser);
	gnome_dialog_button_connect(GNOME_DIALOG(chooser), 1,
		GTK_SIGNAL_FUNC(destroy), chooser);
	
	gtk_widget_show_all(chooser);
}

void add_folder(GtkWidget *widget, gpointer data) {
	GtkWidget *chooser;
	GtkWidget *box;
	GtkWidget *scr_wnd;
	GtkWidget *list;
	
	chooser = gnome_dialog_new(_("Choose Folder Type"),
		GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);

	gnome_dialog_set_parent(GNOME_DIALOG(chooser), GTK_WINDOW(app));
	gnome_dialog_set_default(GNOME_DIALOG(chooser), 0);
		
	box = GNOME_DIALOG(chooser)->vbox;
	
	list = g_fHandler.makeCList(FLAG_SEND|FLAG_RECV);
	
	scr_wnd = gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scr_wnd),
		GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(scr_wnd), list);

	gtk_box_pack_start(GTK_BOX(box), scr_wnd, TRUE, TRUE, 0);

	gnome_dialog_button_connect(GNOME_DIALOG(chooser), 0, 
		GTK_SIGNAL_FUNC(doAdd), list);
	gnome_dialog_button_connect(GNOME_DIALOG(chooser), 0, 
		GTK_SIGNAL_FUNC(destroy), chooser);
	gnome_dialog_button_connect(GNOME_DIALOG(chooser), 1,
		GTK_SIGNAL_FUNC(destroy), chooser);
	
	gtk_widget_show_all(chooser);
}

void doDelete(GtkWidget* widget, gpointer data) {
	GtkWidget **widgets = (GtkWidget**)data;
	GtkCList *list = GTK_CLIST(widgets[1]);
	GtkToggleButton chk = GTK_CHECK_BUTTON(widgets[2])->toggle_button;
	gboolean remove_complete = gtk_toggle_button_get_active(&chk);

	gint row;

	if ((row = get_row(list)) == -1) {
		gnome_warning_dialog_parented(_("Please choose a folder to delete"),
			GTK_WINDOW(widgets[0]));
		return;
	}

	CFolder* folder = (CFolder*)gtk_clist_get_row_data(list, row);

	folder->remove(!remove_complete);
	gtk_widget_destroy(widgets[0]);
}

void delete_folder(GtkWidget *widget, gpointer data) { 
}

void delete_folderM(GtkWidget* widget, gpointer data) {
	GtkWidget* dialog = gnome_dialog_new(_("Please choose a folder to delete"),
		GNOME_STOCK_BUTTON_OK,
		GNOME_STOCK_BUTTON_CANCEL,
		NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));

	GtkBox* box = GTK_BOX(GNOME_DIALOG(dialog)->vbox);

	GtkWidget *foldList = g_folders.makeCList();
	GtkWidget *chkButton = gtk_check_button_new_with_label(_("Delete from disk"));

	gtk_box_pack_start(box, gtk_label_new(_("Please choose a folder to delete")),
		FALSE, FALSE, 0);
	gtk_box_pack_start(box, chkButton, FALSE, FALSE, 5);
	gtk_box_pack_start(box, foldList, TRUE, TRUE, 5);
	GtkWidget** widgets = new GtkWidget*[3];
	widgets[0] = dialog;
	widgets[1] = foldList;
	widgets[2] = chkButton;

	gnome_dialog_button_connect(GNOME_DIALOG(dialog), 0,
		GTK_SIGNAL_FUNC(doDelete), widgets);
	gnome_dialog_button_connect(GNOME_DIALOG(dialog), 1,
		GTK_SIGNAL_FUNC(destroy), dialog);

	gtk_widget_show_all(dialog);
}
