/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/

#ifdef HAVE_CONFIG_H
# include<config.h>
#endif
#include <gnome.h>
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "main.h"
#include "imap.h"

void CFolderImap::error(gchar* err, gchar* line) {
	if (err == NULL) return;

	gchar *text = g_strdup_printf(_("Error while reading mails via IMAP:\n"
		"%s\nServer said: %s\n"),
		err, line ? line : "");

	gnome_app_error(GNOME_APP(app), text);

	g_free(text);
}

PROTO_ERR CFolderImap::login() {

	if (fd != -1) {
		fd_set wfds;
		timeval tv;
		FD_ZERO(&wfds);
		FD_SET(fd, &wfds);

		tv.tv_sec = 0;
		tv.tv_usec = 100;
		if (::select(fd + 1, NULL, &wfds, NULL, &tv) > 0)
			return OK;
		close(fd);
	}
	num = 0;

	gchar cmd[151];
	gchar tag[8];
	gchar reply[301];
	gint ret;

	sprintf(tag, "A%.6i", num++);
	snprintf(cmd, 150, "%s login %s %s\n", tag, user, pw);

	fd = conn(server, port);
	if (fd < 0) return ERR_CONN;

	ret = recvline(fd, reply, 300); // receive greeting
	ret = send(fd, cmd, strlen(cmd), 0); // send command

	ret = recvline(fd, reply, 300); // receive login reply
	while (reply[0] == '*') ret = recvline(fd, reply, 300);

	gchar *result = (reply + strlen(tag) + 1);
	if (strncasecmp(result, "OK", 2) == 0) {
		return OK;
	}
	else {
		return ERR_AUTH;
	}

}

CFolderImap::CFolderImap(const gchar *Nserver, gint Nport, gint Nreserved, const gchar* Nuser, const gchar* Npw, const gchar *Nmailbox) {
	CFolderImap();
	fd = -1;
	server = g_strdup(Nserver);
	port = Nport;
	user = g_strdup(Nuser);
	if (Npw) pw = g_strdup(Npw);
	mailbox = g_strdup(Nmailbox);
}

CFolderImap::CFolderImap() {
	server = user = pw = mailbox = NULL;
	port = 0;
	fd = -1;
	num = 0; 
}

int CFolderImap::select(void* data) {
	ImapMsgData *imp = (ImapMsgData*)data;
	return CFolder::select(get(data), fl, isqp, imp->row, data);
}

int CFolderImap::init(GtkCList *mails, GtkText *display) {
/* [imap]
 * server = 192.168.1.1:143:chb:pw:INBOX
 * provider = imap.provider.net:143:myuser:mypw:mailings
 */
	gpointer handle = gnome_config_init_iterator("/mmc/imap");

	gchar *desc, *server;

	while ( (handle = gnome_config_iterator_next(handle, &desc, &server)) ) {
		gchar *port = strstr(server, ":");
		g_assert(port != NULL);
		*port++ = 0;
		gchar *user = strstr(port, ":");
		g_assert(user != NULL);
		*user++ = 0;
		gchar *pw = strstr(user, ":");
		g_assert(pw != NULL);
		*pw++ = 0;
		gchar *fold = strstr(pw, ":");
		g_assert(fold != NULL);
		*fold++ = 0;

		// variables: server, port, user, pw, fold

		CFolderImap *newfolder = new CFolderImap(server, atoi(port), 0, user, pw, fold);
		newfolder->text = display;
		newfolder->mail_list = mails;
		g_folders.add(newfolder, desc);

		g_free(desc);
		g_free(server);
	}
/*
	CFolderImap *newfolder = new CFolderImap("localhost", 143, 0, "chb", "w647td", "INBOX");
	newfolder->text = display;
	newfolder->mail_list = mails;
	g_folders.add(newfolder, "imap-test");
*/
	return 0;
}

int CFolderImap::parse() {
	PROTO_ERR result = login();
	gchar ans[200];
	gchar tag[9];
	gchar cmd[200];

	switch (result) {
	case OK:
		break;
	case ERR_AUTH:
		error(_("Invalid Username/Password\n"));
		return -1;
		break;
	case ERR_CONN:
		error(_("Connection failed."));
		return -1;
		break;
	case ERR_RETR:
	case ERR_OTHER:
	default:
		break;
	}
        sprintf(tag, "A%.6i", num++);
	snprintf(cmd, 200, "%s SELECT %s\n", tag, mailbox);
	
	send(fd, cmd, strlen(cmd), 0);
	do {
		recvline(fd, ans, 200);
		if (strstr(ans, "EXIST")) {
			gchar *x = strstr((ans+2), " ");
			if (x) *x = 0;
			else {
				g_warning(_("Could not parse server reply."));
				return -1;
			}
			numMsg = atoi(ans+2);
		}
	} while (strncasecmp(ans, tag, strlen(tag)));

	gchar* status = (ans + strlen(tag) + 1);
	if (strncasecmp(status, "BAD", 3) == 0) {
		g_warning(_("Program Bug discovered. Mail mmc-users@lists.sourceforge.net and write:\n"
			"%s%s"), cmd, ans);
		close(fd);
		return -1;
	}
	else if (strncasecmp(status, "NO", 2) == 0) {
		g_print(_("No such mailbox / Can't access mailbox"));
		// TODO: display in GUI.
		return -1;
	}

	for (int i=1; i < numMsg; i++) { // FIXME/TODO this contains some
		                         // implementation bugs.
		gchar *cols[3];
		for (int j = 0; j<3; j++) cols[j]=NULL;

		sprintf(tag, "A%.6i", num++);
		snprintf(cmd, 200, "%s FETCH %i FLAGS\n", tag, i);
		send(fd, cmd, strlen(cmd), 0);
		recvline(fd, ans, 200);
		// ok, we should have got sth. like this:
		// * 8 FETCH (FLAGS (\Seen \Deleted))
		// or:
		// * 8 FETCH (FLAGS (\Seen))

		if (strstr(ans, "\\Deleted")) continue;

		recvline(fd, ans, 200);

		sprintf(tag, "A%.6i", num++);
		snprintf(cmd, 200, "%s FETCH %i BODY[HEADER]\n", tag, i);
		send(fd, cmd, strlen(cmd), 0);
		recvline(fd, ans, 200);
		do {
			recvline(fd, ans, 200);
			if (strncasecmp(ans, "From:", 5) == 0)
				cols[2] = g_strdup(ans + 5);
			else if (strncasecmp(ans, "Date:", 5) == 0)
				cols[1] = g_strdup(ans + 5);
			else if (strncasecmp(ans, "Subject:", 8) == 0)
				cols[0] = g_strdup(ans + 8);
		} while (*ans != ')'); // this is one of the above mentioned bugs!
		*ans = 0;
		recvline(fd, ans, 200);

		gint rnum = gtk_clist_append(mail_list, cols);
		ImapMsgData *dat = new ImapMsgData;
		dat->msgnum = i;
		dat->row = rnum;
		CMessage *msg = new CMessage(this, dat);
		gtk_clist_set_row_data(mail_list, rnum, msg);

		for (int j = 0; j < 3; j++) {
			if (cols[j]) {
				g_free(cols[j]);
			}
		}
	}


	close(fd);

	return 0;

}

int CFolderImap::create() {
#ifdef DEBUG
	folderDialog(TRUE);
#else
	gnome_app_error(GNOME_APP(app), _("Not yet implemented"));
#endif
	return 0;
}

int CFolderImap::addFolder() {
	// need a dialog asking for server,port,user,pw,folder
	// can be the same though as for create
	// hm.. maybe create imapFolderDialog function
	folderDialog(FALSE);
	return 0;
}

void CFolderImap::folderDialog(gboolean create) {
	GtkWidget *dialog = gnome_dialog_new(_("IMAP Folder"),
		GNOME_STOCK_BUTTON_OK,
		GNOME_STOCK_BUTTON_CANCEL,
		NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
	gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
	GtkWidget *box = GNOME_DIALOG(dialog)->vbox;

	GtkWidget *table = gtk_table_new(5, 2, FALSE);

	GtkWidget *desc;
	GtkWidget *_server, *server;
	GtkWidget *_port, *port;
	GtkWidget *_user, *user;
	GtkWidget *pass;
	GtkWidget *folder;

	desc = gtk_entry_new();

	folder = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(folder), DEFAULT_IMAP_FOLDER);

	_server = gnome_entry_new("server");
	server = gnome_entry_gtk_entry(GNOME_ENTRY(_server));

	_port = gnome_entry_new("imap_port");
	port = gnome_entry_gtk_entry(GNOME_ENTRY(_port));
	gtk_entry_set_text(GTK_ENTRY(port), DEFAULT_IMAP_PORT);

	_user = gnome_entry_new("username");
	user = gnome_entry_gtk_entry(GNOME_ENTRY(_user));

	pass = gtk_entry_new();
	gtk_entry_set_visibility(GTK_ENTRY(pass), FALSE);

	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Server")),
		0, 1, 0, 1);
	gtk_table_attach_defaults(GTK_TABLE(table), _server,
		1, 2, 0, 1);

	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Port")),
		0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(table), _port,
		1, 2, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Username")),
		0, 1, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(table), _user,
		1, 2, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Password")),
		0, 1, 3, 4);
	gtk_table_attach_defaults(GTK_TABLE(table), pass,
		1, 2, 3, 4);
	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Folder on Server")),
		0, 1, 4, 5);
	gtk_table_attach_defaults(GTK_TABLE(table), folder,
		1, 2, 4, 5);
	gtk_table_attach_defaults(GTK_TABLE(table), gtk_hseparator_new(),
		0, 2, 5, 6);
	gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(_("Description")),
		0, 1, 6, 7);
	gtk_table_attach_defaults(GTK_TABLE(table), desc,
		1, 2, 6, 7);

	gtk_box_pack_start(GTK_BOX(box), table, TRUE, TRUE, 0);

	gtk_widget_show_all(box);
	if (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0) {
		gchar *cnf_string = g_strdup_printf("%s:%s:%s:%s:%s",
			gtk_entry_get_text(GTK_ENTRY(server)),
			gtk_entry_get_text(GTK_ENTRY(port)),
			gtk_entry_get_text(GTK_ENTRY(user)),
			gtk_entry_get_text(GTK_ENTRY(pass)),
			gtk_entry_get_text(GTK_ENTRY(folder)));
		gnome_config_push_prefix("/mmc/imap/");
		gnome_config_set_string(gtk_entry_get_text(GTK_ENTRY(desc)), cnf_string);
		gnome_config_pop_prefix();
		gnome_config_sync();
		g_free(cnf_string); // TODO IMPORTANT: if create, do create the folder. but later...

		CFolderImap *newfolder = new CFolderImap(
			gtk_entry_get_text(GTK_ENTRY(server)),
			atoi(gtk_entry_get_text(GTK_ENTRY(port))),
			0,
			gtk_entry_get_text(GTK_ENTRY(user)),
			gtk_entry_get_text(GTK_ENTRY(pass)),
			gtk_entry_get_text(GTK_ENTRY(folder)));
		newfolder->text = GTK_TEXT(view_mail);
		newfolder->mail_list = GTK_CLIST(::mail_list);
		g_folders.add(newfolder, gtk_entry_get_text(GTK_ENTRY(desc)));
		gchar* toadd[2];
		toadd[0] = gtk_entry_get_text(GTK_ENTRY(desc));
		toadd[1] = NULL;
		gint count = gtk_clist_append(GTK_CLIST(mboxes), toadd);
		gtk_clist_set_row_data(GTK_CLIST(mboxes), count, newfolder);


	}
	gnome_dialog_close(GNOME_DIALOG(dialog));
}

char* CFolderImap::get(void* data) {
	ImapMsgData *imd = (ImapMsgData*)data;

	guint msgnum = imd->msgnum;
	gchar tag[10];
	gchar cmd[201];
	gchar buf[201];

	if (login() != OK) return NULL; // TODO display error dialog; btw, login should also return server reply

	snprintf(tag, 9, "A%.6i", num++);
	snprintf(cmd, 200, "%s SELECT %s\n", tag, mailbox);
	send(fd, cmd, strlen(cmd), 0);
	recvline(fd, buf, 200); // TODO error checking

	snprintf(tag, 9, "A%.6i", num++);
	snprintf(cmd, 200, "%s FETCH %i RFC822\n", tag, msgnum);
	send(fd, cmd, strlen(cmd), 0);

	recvline(fd, buf, 200);
	while (buf[0] == '*') recvline(fd, buf, 300);

	GString *theMail = g_string_new("");
	do {
		recvline(fd, buf, 200);
		if (*buf != ')') g_string_append(theMail, buf);
	} while (*buf != ')');
	recvline(fd, buf, 200);

	gchar *result = theMail->str;
	g_string_free(theMail, FALSE);
	return result;

}

int CFolderImap::del(void* data) {
	ImapMsgData *imp = (ImapMsgData*)data;

	int msgnum = imp->msgnum;
	login();

	gchar tag[8];
	gchar cmd[151];
	gchar buf[301];

	sprintf(tag, "A%.6i", num++);
	snprintf(cmd, 150, "%s SELECT %s\n", tag, mailbox);
	send(fd, cmd, strlen(cmd), 0);
	do { recvline(fd, buf, 300); } while (buf[0] == '*'); // TODO error checking

	sprintf(tag, "A%.6i", num++);
	snprintf(cmd, 150, "%s STORE %i +FLAGS.SILENT (\\Deleted)\n", tag, msgnum);
	send(fd, cmd, strlen(cmd), 0);

	parse();

	return 0;
}

