/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/
#ifdef HAVE_CONFIG_H
#   include <config.h>
#endif

#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <gnome.h>
#include <libgnome/gnome-i18n.h>
#include <libgnome/gnome-config.h>
#include <libgnome/gnome-popt.h>
#include "protocols.h"
#include "main.h"
#include "msg.h"
#include "pop3.h"
#include "smtp.h"
#include "imap.h"
#include "compose.h"
#include "mime.h"
#include "address.h"
#include "folders.h"
#include "defaults.h"

gchar *g_mmcHome;

GtkWidget *mail_list;


CFolderHandler g_fHandler;
CProtoHandler  g_pHandler;
CFolders       g_folders;

gchar *geometry = NULL;

static struct poptOption options[] = {
	{
		"geometry",
		'\0',
		POPT_ARG_STRING,
		&geometry,
		0,
		N_("Specify the geometry of the main window"),
		N_("WIDTHxHEIGHT+X+Y")
	},
	{ NULL, 0, 0, NULL, 0, NULL, NULL }
};

static GnomeUIInfo file_menu[] = {
	GNOMEUIINFO_ITEM_STOCK(N_("_Get Mail"),N_("Receive Mail from a POP3 Server"),get_mail,GNOME_STOCK_MENU_MAIL_RCV),
	GNOMEUIINFO_ITEM_STOCK(N_("_Compose a new E-Mail"), N_("Opens the message compose window"),compose_msg, GNOME_STOCK_MENU_MAIL_NEW),
	GNOMEUIINFO_MENU_EXIT_ITEM(tidy_up,NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo folder_menu[] = {
	GNOMEUIINFO_ITEM_NONE(N_("_Create new folder"),N_("Creates a new Mailfolder and adds it to the folderlist"), new_folder),
	GNOMEUIINFO_ITEM_NONE(N_("_Add a folder"),N_("Adds an existing folder to the folderlist"), add_folder),
	GNOMEUIINFO_ITEM_NONE(N_("_Delete a folder"),N_("Deletes a folder"), delete_folderM)
};

static GnomeUIInfo msg_menu[] = {
	GNOMEUIINFO_ITEM_NONE(N_("Copy"), N_("Copy the message to another Folder"), copy),
	GNOMEUIINFO_ITEM_NONE(N_("Move"), N_("Move the message to another Folder"), move)

};

static GnomeUIInfo pref_menu[] = {
	GNOMEUIINFO_TOGGLEITEM(N_("_Show all headers"), N_("Shows all headers for the current E-Mail"),show_all_head,NULL),
	GNOMEUIINFO_TOGGLEITEM(N_("_Decode using Quoted-Printable"), N_("Attempts to decode the message as if it was encoded with Quoted-Printable"), is_qp,NULL),
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(show_prefs,NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu[] = {
	GNOMEUIINFO_MENU_ABOUT_ITEM(create_about,NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar[] = {
	GNOMEUIINFO_MENU_FILE_TREE(file_menu),
	GNOMEUIINFO_SUBTREE_HINT(N_("_Folders"), N_("Commands for the folderlist"), folder_menu),
	GNOMEUIINFO_SUBTREE_HINT(N_("_Message"), N_("Commands for the Message"), msg_menu),
	GNOMEUIINFO_MENU_SETTINGS_TREE(pref_menu),
	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo toolbar[] = {
	GNOMEUIINFO_ITEM_STOCK(N_("Get Mail"),N_("Receive Mail from a POP3 Server"),get_mail,GNOME_STOCK_PIXMAP_MAIL_RCV),
	GNOMEUIINFO_ITEM_STOCK(N_("New E-Mail"), N_("Opens the message compose window"), compose_msg, GNOME_STOCK_PIXMAP_MAIL_NEW),
	GNOMEUIINFO_ITEM_STOCK(N_("Reply"), N_("Replys to the currently marked message"), reply_to, GNOME_STOCK_PIXMAP_MAIL_RPL),
	GNOMEUIINFO_SEPARATOR,
#ifdef DEBUG
	GNOMEUIINFO_ITEM_STOCK(N_("Search"), N_("Searches for a text in the Folder(s)"), search, GNOME_STOCK_PIXMAP_SEARCH),
#endif
/*	GNOMEUIINFO_ITEM_NONE(N_("Copy"), N_("Copy the message to another Folder"), copy),
	GNOMEUIINFO_ITEM_NONE(N_("Move"), N_("Move the message to another Folder"), move),
*/	GNOMEUIINFO_ITEM_STOCK(N_("Delete"), N_("Deletes the current Message"), del, GNOME_STOCK_PIXMAP_TRASH), 
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK(N_("Attachments"), N_("Displays attachments of the current message"), attachm, GNOME_STOCK_PIXMAP_ATTACH),
	GNOMEUIINFO_ITEM_STOCK(N_("Address Book"), N_("Display user's address book"), open_book, GNOME_STOCK_PIXMAP_BOOK_RED),
//	GNOMEUIINFO_ITEM_STOCK(N_("Exit"),N_("Exit MMC"),tidy_up,GNOME_STOCK_PIXMAP_EXIT),
	GNOMEUIINFO_END
};

static GnomeUIInfo folder_popup[] = {
	GNOMEUIINFO_ITEM_NONE(N_("_Create new folder"), N_("Creates a new Mailfolder and adds it to the folderlist"), new_folder),
	GNOMEUIINFO_ITEM_NONE(N_("_Add a folder"),N_("Adds an existing folder to the folderlist"), add_folder),
	GNOMEUIINFO_ITEM_NONE(N_("_Delete the folder"), N_("Deletes the folder"), delete_folder),
	GNOMEUIINFO_END
};
 

GtkWidget *app;
GtkWidget *main_paned;
GtkWidget *view_mail;

GtkWidget *box_for_mail;
GtkWidget *mail_scroll;
GtkWidget *list_scroll;
GtkWidget *box_right;
GtkWidget *mboxes;
GtkWidget *status;

#ifdef DEBUG
//==============REMOVE SOON==============
// THIS IS JUST DEBUG CODE WHICH ONLY WORKS ON MY (Biesi's) PC
void test() {
	FILE *file=fopen("/home/chb/simdl.htm","r");
	gchar buf[2000];
	gchar *ptr=buf;
	while (!feof(file)) {
		fgets(ptr,50,file);
		ptr+=strlen(ptr);
	}

g_print("SIZE:%i\n",ptr-buf);
	encode_b64(buf, ptr-buf);
g_print("\n");

	abort();
}
//==============END REMOVE===============
#endif


int main(int argc, char *argv[]) {
#ifdef DEBUG
	if (argc>1) if ( *argv[1]=='D') test(); //2nd part of Debug code above
#endif
	poptContext pctx;
	gchar *list_titles[] = {
	N_("Subject"),
	N_("Date"),
	N_("From")};
	
	if (!getenv("HOME") || !*(getenv("HOME"))) g_error(_("The HOME environment variable MUST be set!\n"));
	g_mmcHome=new gchar[strlen(getenv("HOME"))+6];
	sprintf(g_mmcHome,"%s" MMC_DIR,getenv("HOME"));
	mkdir(g_mmcHome,0700);

	gnome_init_with_popt_table (PACKAGE,VERSION,argc,argv,options,0,&pctx);
	poptFreeContext(pctx);
	
	gnome_dns_init(2);
	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
	textdomain(PACKAGE);

	app = gnome_app_new(PACKAGE, _("My Mail Client"));
	gtk_signal_connect(GTK_OBJECT(app),"delete_event",
		GTK_SIGNAL_FUNC(tidy_up), NULL);

	main_paned=gtk_hpaned_new();
	box_right=gtk_vpaned_new();

	status=gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_USER);

	gnome_appbar_set_default(GNOME_APPBAR(status), "MMC " VERSION);
	
	gnome_app_create_menus(GNOME_APP (app), menubar);
	gnome_app_create_toolbar(GNOME_APP (app), toolbar);
	gnome_app_set_statusbar(GNOME_APP (app), status);

	gnome_app_install_menu_hints(GNOME_APP (app), menubar);

	view_mail=gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT(view_mail), FALSE);
	gtk_text_set_word_wrap(GTK_TEXT(view_mail) ,TRUE);
	mail_scroll=gtk_vscrollbar_new (GTK_TEXT(view_mail)->vadj);

	mail_list=gtk_clist_new(3);
	for (int i=0; i<3; i++)
		gtk_clist_set_column_title(GTK_CLIST(mail_list),i,_(list_titles[i]));
	
	gtk_clist_column_titles_show(GTK_CLIST(mail_list));

	gtk_clist_set_selection_mode(GTK_CLIST(mail_list), GTK_SELECTION_SINGLE);
	gtk_clist_column_titles_active(GTK_CLIST(mail_list));
	
	gtk_clist_set_column_width(GTK_CLIST(mail_list), 0, 200);
	gtk_clist_set_column_width(GTK_CLIST(mail_list), 1, 100);

	gtk_signal_connect(GTK_OBJECT(mail_list), "select_row", GTK_SIGNAL_FUNC(select_mail), NULL);

	// Initialize Folder Handler
	CFolder *fileFolder = new CFolderFile;
	g_fHandler.add(fileFolder, GTK_CLIST(mail_list), GTK_TEXT(view_mail));
	CFolder *imapFolder = new CFolderImap;
	g_fHandler.add(imapFolder, GTK_CLIST(mail_list), GTK_TEXT(view_mail));

	mboxes = g_folders.makeCList();

	GtkWidget *popup = gnome_popup_menu_new(folder_popup);
	gnome_popup_menu_attach(popup, mboxes, NULL);

	// Following Line is for Sorting
	gtk_signal_connect(GTK_OBJECT(mail_list), "click_column",
		GTK_SIGNAL_FUNC(do_sort), NULL);

	gtk_signal_connect(GTK_OBJECT(mboxes), "select-row",
		GTK_SIGNAL_FUNC(select_box), mboxes);

	list_scroll=gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(list_scroll),
		GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(list_scroll), mail_list);

	// Initializing protocol Handlers
	CProto *pop3 = new CProtoPop3;
	g_pHandler.add(pop3);
	CProto *smtp = new CProtoSmtp;
	g_pHandler.add(smtp);

	box_for_mail=gtk_hbox_new(FALSE,0);
	gtk_paned_pack1(GTK_PANED(box_right),list_scroll,TRUE,TRUE);
	gtk_paned_pack2(GTK_PANED(box_right),box_for_mail,TRUE,TRUE);

	gtk_paned_pack1(GTK_PANED(main_paned),mboxes,TRUE,TRUE);
	gtk_paned_pack2(GTK_PANED(main_paned),box_right,TRUE,TRUE);

	gtk_box_pack_start(GTK_BOX(box_for_mail),view_mail,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(box_for_mail),mail_scroll,FALSE,FALSE,0);

	gnome_app_set_contents(GNOME_APP(app),main_paned);

	if (!geometry) {
		geometry=gnome_config_get_string("/mmc/sizes/geometry=" DEFAULT_SIZE );
	}
	else geometry=g_strdup(geometry);

	gint x,y,w,h;
	if (gnome_parse_geometry(geometry, &x,&y,&w,&h)) {
		if ((x != -1) && (y != -1))
			gtk_widget_set_uposition(app,x,y);
		if ((w != -1) && (h != -1))
			gtk_window_set_default_size(GTK_WINDOW(app), w, h);
	}
	else g_error(_("Could not parse geometry string '%s'!\n"),geometry);
	
	g_free(geometry);
	
	
 
	gtk_widget_show_all(app);

	gtk_main();

	return 0;
}

void tidy_up(GtkWidget *widget, gpointer data2) {
	gchar *geo=gnome_geometry_string(app->window);
	gnome_config_set_string("/mmc/sizes/geometry",geo);
	g_free(geo);

	gnome_config_sync();

	delete[] g_mmcHome;

	gtk_clist_freeze(GTK_CLIST(mail_list));
	gtk_clist_clear(GTK_CLIST(mail_list));
	gtk_main_quit();
}


GtkWidget *add_file;

GtkWidget *font;
GtkWidget *headColor;
GtkWidget *attachColor;
GtkWidget *unreadColor;
GtkWidget *quoteColor;
GtkWidget *protocolR;
GtkWidget *protocolS;
GtkWidget *config_widget;
GtkWidget *config_send;

// property_apply - is called when the user presses OK or Apply
static void property_apply(GnomePropertyBox *box, gint page_num, gpointer data) {

	if (page_num!=-1) return;

	gchar *protname = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(protocolR)->entry));
	gchar *sendname = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(protocolS)->entry));
	CProto *recvProto = g_pHandler.get(protname);
	gchar *idstring = recvProto->applyConfig(FLAG_RECV, config_widget);
	gchar *sndstring = g_pHandler.get(sendname)->applyConfig(FLAG_SEND, config_send);

	gchar *cnf_string = new gchar[310];
	snprintf(cnf_string, 309, "%s@@%s@@%s@@%s", protname, idstring, sendname, sndstring);
	
	gnome_config_set_string("/mmc/accounts/current","default");
	gnome_config_set_string("/mmc/accounts/default",cnf_string);

	delete[] cnf_string;
	recvProto->free(idstring);
	
	gnome_config_set_string("/mmc/advanced/addressfile",
		gtk_entry_get_text(
			GTK_ENTRY(gnome_file_entry_gtk_entry(
				GNOME_FILE_ENTRY(add_file)
				))));

	gushort r,g,b,a;
	gnome_color_picker_get_i16(GNOME_COLOR_PICKER(headColor),
		&r,&g,&b,&a);
	gnome_config_set_int("/mmc/colors/head.red",r);
	gnome_config_set_int("/mmc/colors/head.green",g);
	gnome_config_set_int("/mmc/colors/head.blue",b);

	gnome_color_picker_get_i16(GNOME_COLOR_PICKER(attachColor),
		&r,&g,&b,&a);
	gnome_config_set_int("/mmc/colors/attach.red",r);
	gnome_config_set_int("/mmc/colors/attach.green",g);
	gnome_config_set_int("/mmc/colors/attach.blue",b);

	gnome_color_picker_get_i16(GNOME_COLOR_PICKER(unreadColor),
		&r,&g,&b,&a);
	gnome_config_set_int("/mmc/colors/unread.red",r);
	gnome_config_set_int("/mmc/colors/unread.green",g);
	gnome_config_set_int("/mmc/colors/unread.blue",b);

	gnome_color_picker_get_i16(GNOME_COLOR_PICKER(quoteColor),
		&r,&g,&b,&a);
	gnome_config_set_int("/mmc/colors/quote.red",r);
	gnome_config_set_int("/mmc/colors/quote.green",g);
	gnome_config_set_int("/mmc/colors/quote.blue",b);

	
	gnome_config_set_string("/mmc/fonts/mail",
		gnome_font_picker_get_font_name(
			GNOME_FONT_PICKER(font)));

	gnome_config_sync();


}

// entry_changed - makes the apply button sensitive
void entry_changed (GtkWidget *widget, gpointer data) {
	GtkWidget *prefs=(GtkWidget*)data;
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs));
}

void font_changed(GnomeFontPicker *widget, gchar *font_name) {
	GtkWidget *prefs=gtk_widget_get_ancestor(GTK_WIDGET(widget),
			gtk_type_from_name("GnomePropertyBox"));
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs));
}

void prot_changed(GtkWidget *widget, gpointer data) {
#ifdef DEBUG
	gint flags = GPOINTER_TO_INT(data);
	gchar *protname = gtk_entry_get_text(GTK_ENTRY(widget));

	g_print(protname);
#endif
}
/* def_col:
 * Reverts the colors to their default values
 * (header and unread color is blue, attachment
 * color is green)
 */
void def_col(GtkWidget *widget, gpointer bar) {
	gnome_color_picker_set_i16(GNOME_COLOR_PICKER(headColor),
		atol(DEFAULT_HEAD_RED),atol(DEFAULT_HEAD_GREEN),atol(DEFAULT_HEAD_BLUE),0);
	gnome_color_picker_set_i16(GNOME_COLOR_PICKER(attachColor),
		atol(DEFAULT_ATTACH_RED),atol(DEFAULT_ATTACH_GREEN),atol(DEFAULT_ATTACH_BLUE),0);
	gnome_color_picker_set_i16(GNOME_COLOR_PICKER(unreadColor),
		atol(DEFAULT_UNREAD_RED),atol(DEFAULT_UNREAD_GREEN),atol(DEFAULT_UNREAD_BLUE),0);
	gnome_color_picker_set_i16(GNOME_COLOR_PICKER(quoteColor),
		atol(DEFAULT_QUOTE_RED),atol(DEFAULT_QUOTE_GREEN),atol(DEFAULT_QUOTE_BLUE),0);

	GtkWidget *prefs=gtk_widget_get_ancestor(GTK_WIDGET(widget),
		gtk_type_from_name("GnomePropertyBox"));
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prefs));
			
}

void show_prefs(GtkWidget* widget, gpointer data) {

	GtkWidget *prefs;
	GtkWidget *fixed, *adv, *fixedFont, *fixedColor;
	GtkWidget *gAddrFile;

	GtkWidget *default_colors;

	prefs=gnome_property_box_new();

	adv=gtk_fixed_new();
	add_file=gnome_file_entry_new("addressbook",_("Choose an Addressbook file"));
	gtk_fixed_put(GTK_FIXED(adv),gtk_label_new(_("Addressbook file:")), 8, 8);
	gtk_fixed_put(GTK_FIXED(adv),add_file, 120, 8);
	gAddrFile=gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(add_file));

	gchar *add_filename=gnome_config_get_string("/mmc/advanced/addressfile=");
	gtk_entry_set_text(GTK_ENTRY(gAddrFile), add_filename);
	g_free(add_filename);
			

	fixedFont=gtk_fixed_new();
	font=gnome_font_picker_new();
	gnome_font_picker_set_title(GNOME_FONT_PICKER(font), _("Select a Mailfont"));
	gnome_font_picker_set_mode(GNOME_FONT_PICKER(font), GNOME_FONT_PICKER_MODE_FONT_INFO);
	gnome_font_picker_fi_set_use_font_in_label(GNOME_FONT_PICKER(font), TRUE, 14);
	gnome_font_picker_fi_set_show_size(GNOME_FONT_PICKER(font), TRUE);
	gchar *fontname=gnome_config_get_string("/mmc/fonts/mail=" DEFAULT_MAIL_FONT);
	gnome_font_picker_set_font_name(GNOME_FONT_PICKER(font), fontname);
	
	gtk_fixed_put(GTK_FIXED(fixedFont),gtk_label_new(_("Mailfont")),8,8);
	gtk_fixed_put(GTK_FIXED(fixedFont), font, 120, 8);
	
	gtk_signal_connect(GTK_OBJECT(font), "font-set", GTK_SIGNAL_FUNC(font_changed), prefs);
	g_free(fontname);

	fixedColor = gtk_fixed_new();

	// Color Picker: Headers
	headColor = gnome_color_picker_new();

	gushort r,g,b;
	gnome_config_push_prefix("/mmc/colors/");
	r=gnome_config_get_int("head.red=" DEFAULT_HEAD_RED);
	g=gnome_config_get_int("head.green=" DEFAULT_HEAD_GREEN);
	b=gnome_config_get_int("head.blue=" DEFAULT_HEAD_BLUE);
	gnome_color_picker_set_i16(GNOME_COLOR_PICKER(headColor),
		r,g,b,0);
	gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(headColor),
		FALSE);
	gnome_color_picker_set_title(GNOME_COLOR_PICKER(headColor),
		_("Pick Header Color"));

	// Color Picker: Attachments
	attachColor = gnome_color_picker_new();

	r=gnome_config_get_int("attach.red=" DEFAULT_ATTACH_RED);
	g=gnome_config_get_int("attach.green=" DEFAULT_ATTACH_GREEN);
	b=gnome_config_get_int("attach.blue=" DEFAULT_ATTACH_BLUE);
	gnome_color_picker_set_i16(GNOME_COLOR_PICKER(attachColor),
		r,g,b,0);
	gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(attachColor),
		FALSE);

	// Color Picker: Unread Messages
	unreadColor = gnome_color_picker_new();

	r=gnome_config_get_int("unread.red=" DEFAULT_UNREAD_RED);
	g=gnome_config_get_int("unread.green=" DEFAULT_UNREAD_GREEN);
	b=gnome_config_get_int("unread.blue=" DEFAULT_UNREAD_BLUE);
	gnome_color_picker_set_i16(GNOME_COLOR_PICKER(unreadColor),
		r,g,b,0);

	gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(unreadColor),
		FALSE);

	// Color Picker: Quoted Text
	quoteColor = gnome_color_picker_new();

	r=gnome_config_get_int("quote.red=" DEFAULT_QUOTE_RED);
	g=gnome_config_get_int("quote.green=" DEFAULT_QUOTE_GREEN);
	b=gnome_config_get_int("quote.blue=" DEFAULT_QUOTE_BLUE);
	gnome_color_picker_set_i16(GNOME_COLOR_PICKER(quoteColor),
		r,g,b,0);

	gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(quoteColor),
		FALSE);

	
	gnome_config_pop_prefix();

	// Button for default colors
	default_colors=gtk_button_new_with_label(_("Default Colors"));
	gtk_signal_connect(GTK_OBJECT(default_colors), "clicked", GTK_SIGNAL_FUNC(def_col), NULL);
	
	// Put that stuff in a GtkFixed
	gtk_fixed_put(GTK_FIXED(fixedColor), gtk_label_new(_("Header Color")), 8,8);
	gtk_fixed_put(GTK_FIXED(fixedColor), headColor, 170,8);
	gtk_fixed_put(GTK_FIXED(fixedColor), gtk_label_new(_("Attachment Color")), 8, 32);
	gtk_fixed_put(GTK_FIXED(fixedColor), attachColor, 170, 32);
	gtk_fixed_put(GTK_FIXED(fixedColor), gtk_label_new(_("Unread Messages Color")), 8, 56);
	gtk_fixed_put(GTK_FIXED(fixedColor), unreadColor, 170, 56);
	gtk_fixed_put(GTK_FIXED(fixedColor), gtk_label_new(_("Quoted Text Color")), 8,80);
	gtk_fixed_put(GTK_FIXED(fixedColor), quoteColor, 170, 80);
	
	gtk_fixed_put(GTK_FIXED(fixedColor), default_colors, 8, 104);

	gtk_signal_connect(GTK_OBJECT(headColor), "color-set", GTK_SIGNAL_FUNC(font_changed), NULL);
	gtk_signal_connect(GTK_OBJECT(attachColor), "color-set", GTK_SIGNAL_FUNC(font_changed), NULL);
	gtk_signal_connect(GTK_OBJECT(unreadColor), "color-set", GTK_SIGNAL_FUNC(font_changed), NULL);

	GtkWidget *sep1, *sep2; // separator

	sep1 = gtk_hseparator_new();
	sep2 = gtk_hseparator_new();
	gtk_widget_set_usize(sep1, 600, 2);
	gtk_widget_set_usize(sep2, 600, 2);

	fixed=gtk_fixed_new();
	protocolR = g_pHandler.makeCombo(FLAG_RECV);
	protocolS = g_pHandler.makeCombo(FLAG_SEND);

	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(protocolR)->entry), "changed",
			GTK_SIGNAL_FUNC(prot_changed), GINT_TO_POINTER(FLAG_RECV));
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(protocolS)->entry), "changed",
			GTK_SIGNAL_FUNC(prot_changed), GINT_TO_POINTER(FLAG_SEND));

	gtk_fixed_put(GTK_FIXED(fixed), gtk_label_new(_("Receive Protocol")), 0, 0);
	gtk_fixed_put(GTK_FIXED(fixed), protocolR, 150, 0);

	gtk_fixed_put(GTK_FIXED(fixed), sep1, 8, 100);

	gtk_fixed_put(GTK_FIXED(fixed), gtk_label_new(_("Send Protocol")), 0, 105);
	gtk_fixed_put(GTK_FIXED(fixed), protocolS, 150, 105);

	// display a config_widget...
	gchar *protName = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(protocolR)->entry));
	config_widget = g_pHandler.get(protName)->config(FLAG_RECV, "", "", prefs);

	gtk_fixed_put(GTK_FIXED(fixed), sep2, 8, 25);
	gtk_fixed_put(GTK_FIXED(fixed), config_widget, 8, 30);

	gchar *sendName = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(protocolS)->entry));
	config_send = g_pHandler.get(sendName)->config(FLAG_SEND, "", "", prefs);

	gtk_fixed_put(GTK_FIXED(fixed), config_send, 8, 130);

	gtk_signal_connect(GTK_OBJECT(prefs),"apply",GTK_SIGNAL_FUNC(property_apply),
		NULL);

	gtk_signal_connect(GTK_OBJECT(gAddrFile),"changed", GTK_SIGNAL_FUNC(entry_changed), prefs);

	gnome_property_box_append_page(GNOME_PROPERTY_BOX(prefs),
		fixed, gtk_label_new(_("General")));
	gnome_property_box_append_page(GNOME_PROPERTY_BOX(prefs),
		adv, gtk_label_new(_("Advanced")));
	gnome_property_box_append_page(GNOME_PROPERTY_BOX(prefs),
		fixedFont, gtk_label_new(_("Fonts")));
	gnome_property_box_append_page(GNOME_PROPERTY_BOX(prefs),
		fixedColor, gtk_label_new(_("Colors")));

	gtk_widget_show_all(prefs);
	gnome_dialog_run(GNOME_DIALOG(prefs));
}


void create_about(GtkWidget *widget, gpointer data) {

	const gchar *authors[] = {
		"Christian Biesinger",
		"Stephen Behnke",
		NULL
	};

	gtk_widget_show_all (gnome_about_new(PACKAGE, VERSION, "(c) 2000-2001 by Christian Biesinger and Stephen Behnke",
		authors, _("Homepage: http://mmc.sourceforge.net\nCompiled on: " __DATE__ " " __TIME__), PIXMAPDIR "mmc-logo.png"));	
}

