/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <ctype.h>
#include <memory.h>
#include <string.h>
#include "mime.h"
#include "main.h"
#include "msg.h"

char base64table[67]={
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=\0"
};

/*
   char hex_to_char(char *hex);

   Interprets the given string as a 2-digit hexadecimal number and converts it to a char.

   Example: "FC" becomes (char)253 = 
*/
char hex_to_char(char *hex) {
	char code[3];
	unsigned int  dec;
	snprintf(code,3,"%s",hex);
	sscanf(code,"%x",&dec);
	return (char)dec;
}

/*
   char *decode_mime(char *line);

   Decodes a line which contains encoded characters - encoding must be Quoted-Printable (RFC 1341).

   Returns: the decoded line (NOT newly allocated)
*/
char *decode_mime(char *line) {
	if (!line) return NULL;
	char *ptr=line;

	while (*ptr) {
		if (*ptr == '=') {
			if (*(ptr+1)=='\n') { *ptr=0; break; }
			*ptr=hex_to_char(ptr+1);
			for (char *vptr=(ptr+1); *(vptr+1); vptr++) *vptr=*(vptr+2);
		}
		ptr++;
	}
	return line;
}

/*
   char *decode_subj(char *line);

   Decodes the Subject of an email message, if it was encoded using Quoted-Printable. (RFC 1341)
   For example:
     Testing german umlauts =?ISO-8859-1?Q?=FC?=
   will become
     Testing german umlauts 

   Returns: the decoded Subject, which is NOT newly allocated, but the modified 1st parameter.
*/
char *decode_subj(char *line) {
	char *ptr=line;
	while(*ptr) {
		if (*ptr=='=') {
			char *up=ucase(ptr);
			int dist=15;
			if (memcmp(up,"=?ISO-8859-1?Q?",15)==0) { // subject is quoted-printable
				for (; (*(ptr+dist-1)); ptr++) {
					if (memcmp((ptr+dist),"?=",2)==0) dist+=2;
					*ptr=*(ptr+dist);
					if (*ptr=='_') *ptr=' ';
				}
				
			}
			line=decode_mime(line);
			g_free(up);
		}
		ptr++;
	}

	return line;
}

/*
   short lookup_code(char code);

   Looks up the Number of the given character in the Base64 Alphabet.
   
   Returns: the Number of the character.
*/
short lookup_code(char which) {
	for (gushort i=0; i<sizeof(base64table); i++)
		if (base64table[i]==which) return i;
	return -1;
}

/*
   char *decode_b64(char *encoded);

   Decodes a byte stream, which was encoded using the Base64 encoding, as defined in RFC 1341.

   Returns: Decoded Byte stream, which should be g_freed, or better delete[]ed later on.
*/
char *decode_b64(const char *encoded, guint *length) {
	gboolean del_length=FALSE;
	if (length==NULL) {
		length=new guint;
		del_length=TRUE;
	}
	*length=0;
	int size=(int)(strlen(encoded)*1.36+3);
	char *out2=new char[size];
	char *out=out2;
	char workin[4], workout;

	while (*encoded) {
		// I must fetch 4 Byte = 24 Bit according to Base64
		// thus will be 3 Byte
		for (int i=0; i<4; i++) {
			workin[i]=lookup_code(*encoded++);
			if (workin[i]<0) { i--; continue; }
		}

		// 1. Stream bit is "left" bit in output
		if (workin[0]<64 && workin[1]<64) {
			workout=workin[0]<<2;
			workout|=((workin[1]&48)>>4);
			(*length)++;
			*out++=workout;
		}
		else break;

		if (workin[1]<64 && workin[2]<64) {
			workout=(workin[1]&15)<<4;
			workout|=(workin[2]&60)>>2;
			(*length)++;
			*out++=workout;
		}
		else break;
		
		if (workin[2]<64 && workin[3]<64) {
			workout=(workin[2]&3)<<6;
			workout|=(workin[3]);
			(*length)++;
			*out++=workout;
		}
		else break;
	}
	if (del_length) delete length;
	*out=0;
	return out2;
}

/*
char *encode_b64(char *str);

Encodes a given byte stream using the Base64 decoding as defined in RFC 1341.

Returns: Pointer to the encoded Stream.
Should be g_freed, or better delete[]ed, later on.
*/
gchar *encode_b64(const gchar *str, guint len) {
	char workin[3], workout[4];
	long size=(int)(strlen(str)*0.77+3);
	short tmp;
	char *out=new char[size];
	char *ptr=out;
//TODO FIND OUT WTF THIS "gboolean LAST" WAS MEANT FOR
//	gboolean LAST=FALSE;
	gint line=0;
	guint done=0, outp=0;

	while (done<=len) {
		for (int i=0; i<3; i++) {
			workin[i]=*str++;
		}
		workout[0]=base64table[(workin[0]&252)>>2];
		line++; outp++;

		tmp=((workin[0]&3)<<4);
		tmp|=((workin[1]&240)>>4);
		workout[1]=base64table[tmp];
		line++; outp++;
		
		tmp=((workin[1]&15)<<2);
		tmp|=((workin[2]&192)>>6);
		workout[2]=base64table[tmp];
		line++; outp++;

		workout[3]=base64table[workin[2]&63];
		line++; outp++;

		memcpy(ptr,workout,4);
		ptr+=4;

		for (guint x=1; x<=3; x++) {
			done++;
			if (done>len) {
				*(ptr-4+x)=0;
				break;
			}
		}
				
		
		if (line==60){
		       	*ptr++='\n';
			line=0;
		}
	}

	for (guint x=0; x<outp; x++) g_print("%c",out[x]);
	return out;
}

