/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/
#ifdef HAVE_CONFIG_H
#   include <config.h>
#endif
#include <ctype.h>
#include <gnome.h>
#include <string.h>
#include <errno.h>
#include <string>
#include <sys/file.h>
#include "main.h"
#include "msg.h"
#include "mime.h"
#include "compose.h"
#include "defaults.h"

const gchar * welcome =  N_("From aa@bb.com 15 Feb 2000 22:00:00 +0100\n"
			"X-MMC: 0\n"
			"From: Christian Biesinger <biesi@users.sourceforge.net>\n"
			"To: MMC User <mmc-user@localhost>\n"
			"Subject: Welcome to MMC!\n"
			"Date: Fri, 18 Feb 2000 17:01:00 +0100\n\n"
			"Welcome to MMC, the latest and greatest E-Mail client available on the web!\n"
			"*g*\n\n"
			"Currently, it surely misses many features; but this will change.\n\n"
			"This Project was started at the beginning of February 2000.\n\n"
			"Have a lot of fun!\n\n\n"
			"Also note that this program is distributed under the GPL, the GNU General\n"
			"Public License. See the file COPYING for details.\n\n\n"

);


gchar * empty[] = { NULL, NULL, NULL };
GdkColor *COL_UNREAD_F=new GdkColor;
gboolean fl=0, isqp=FALSE;

ATTACHMENTS attach;

GtkWidget *attach_window;
GtkWidget *file_entry;

guint sort_col;
gboolean ascending;
gint datesort(GtkCList*,gconstpointer,gconstpointer);

gint get_row(GtkCList *list) {
	if (list->selection) {
		return GPOINTER_TO_INT(list->selection->data);
	}
	else return -1;
}

CMessage::CMessage() {
	data=folder=NULL;
	from=replyto=to=subj=date=cc=bcc=body=NULL;
	additional=g_string_new("");
}

CMessage::CMessage(CFolder* fold, void *dat) {
	folder=fold;
	data=dat;
	from=replyto=to=subj=date=cc=bcc=body=NULL;
	additional = g_string_new("");
}

CMessage::CMessage(gchar *Nfrom, gchar *Nreplyto, gchar *Nto, gchar *Ncc, gchar *Nbcc,
		gchar *Nsubject, gchar *Nadditional, gchar *Nbody) {
	// parameters begin with N=New (to avoid conflicts with members)
	
	if (Nfrom) from = g_strdup(Nfrom);
	else from=NULL;

	if (Nreplyto) replyto = g_strdup(Nreplyto);
	else replyto = NULL;

	if (Nto) to = g_strdup(Nto);
	else to = NULL;
	
	if (Ncc) cc=g_strdup(Ncc);
	else cc = NULL;

	if (Nbcc) bcc=g_strdup(Nbcc);
	else bcc = NULL;

	if (Nsubject) subj=g_strdup(Nsubject);
	else subj = NULL;

	if (Nadditional) additional = g_string_new(Nadditional);
	else additional = g_string_new("");

	if (Nbody) body = g_strdup(Nbody);
	else body = NULL;
	
	data=NULL;
	folder=NULL;
}

CMessage::~CMessage() {
	if (data) folder->free(data);

	if (from) g_free(from);
	if (replyto) g_free(replyto);
	if (to) g_free(replyto);
	if (cc) g_free(cc);
	if (bcc) g_free(bcc);
	if (subj) g_free(subj);
	g_string_free(additional, TRUE);
	if (body) g_free(body);
}

void CMessage::select() {
	folder->select(data);
}

void CMessage::del() {
	gtk_editable_delete_text(GTK_EDITABLE(view_mail), 0, -1);
	gtk_clist_unselect_row(GTK_CLIST(mail_list), get_row(GTK_CLIST(mail_list)), 0);
	gnome_appbar_push(GNOME_APPBAR(status),_("Deleting/Moving Message..."));
	while (gtk_main_iteration_do(FALSE));
	if (folder->del(data) == 0)
		gnome_app_message(GNOME_APP(app), _("Message deleted."));
	gnome_appbar_pop(GNOME_APPBAR(status));
}


gchar *h(const gchar *msg, const gchar *head) {
	gchar *line;
	gchar *res = NULL;
	line = getHead(msg, head);
	if (line) {
		res = g_strdup(line + strlen(head) + 1);
		delete[] line;
	}

	return res;
}

void CMessage::fill() {
	gchar *msg = getRaw();

	from = h(msg, "From:");
	to = h(msg, "To:");
	cc = h(msg, "CC:");
	bcc = h(msg, "BCC:");
	subj = h(msg, "Subject:");
	date = h(msg, "Date:");
	replyto = h(msg, "Reply-To:");
	msgid = h(msg, "Message-Id:");

	gchar *x = strstr(msg, "\n\n");
	if (x) body = g_strdup(x+2);

	g_free(msg);
}

int CFolderFile::init(GtkCList *mails, GtkText *display) {
	// I'll read the config string here & build the list

	gchar *toadd[]={NULL,NULL};
	guint count=0;
	gchar *ptr, *ptr2, *nameptr, *nameptr2;
	gchar *box_list=gnome_config_get_string
		("/mmc/General/MailboxesTitles=" DEFAULT_FOLDERS_TITLE);
	gchar *box_names=gnome_config_get_string
		("/mmc/General/MailboxesFilenames=" DEFAULT_FOLDERS_FILE);
	// Note that {MD} is the Directory of the MMC Mailboxes,
	// usually ~/.mmc/
	// ~ is the user's Home Directory
	
	nameptr=box_names;

	ptr2=box_list;
	while ((ptr=strstr(ptr2,":"))) {
		ptr++;
		ptr2=ptr;

		gchar *end;
		guint len;
		end=strstr(ptr2,":");
		
		if (end)
			len=end-ptr2;
		else
			len=strlen(ptr2);

		toadd[0]=new gchar[len+1];
		memcpy(toadd[0],ptr2,len);
		toadd[0][len]=0;
		gchar *desc = toadd[0];

		nameptr=strstr(nameptr,":")+1;
		nameptr2=strstr(nameptr,":");
		if (nameptr2)
			len=nameptr2-nameptr;
		else
			len=strlen(nameptr);

		toadd[0]=new gchar[len+1];
		memcpy(toadd[0],nameptr,len);
		toadd[0][len]=0;

		CFolderFile *f=new CFolderFile(getName(*toadd), toadd[0], desc, mails, display);
		// Add the Folder to the global Folder list
		g_folders.add(f, desc);
		count++;

		delete toadd[0];
	}

	g_free(box_list);
	g_free(box_names);
	
	return count;

}

CFolderFile::CFolderFile(gchar *fname, gchar *savedname, gchar *desc, GtkCList *list, GtkText *disp) {
	filename = fname;
	text = disp;
	mail_list = list;
	description = desc;
	saved_name = g_strdup(savedname);
}

CFolderFile::CFolderFile() {
	saved_name = NULL;
}

CFolderFile::~CFolderFile() {
	if (saved_name) g_free(saved_name);
}

int CFolderFile::parse() {
	long num_msg=-1;
	FILE* mails;
	gulong unread=0;
	
	if (filename==NULL || *filename==0) {
		g_warning("No Filename specified!\n");
		return 0;
	}

	gnome_appbar_pop(GNOME_APPBAR(status));

	sort_col=0;
	ascending=TRUE;

	COL_UNREAD_F->red=gnome_config_get_int("/mmc/colors/unread.red=" DEFAULT_UNREAD_RED);
	COL_UNREAD_F->green=gnome_config_get_int("/mmc/colors/unread.green=" DEFAULT_UNREAD_GREEN);
	COL_UNREAD_F->blue=gnome_config_get_int("/mmc/colors/unread.blue=" DEFAULT_UNREAD_BLUE);


	mails=fopen(filename,"r");
	if (mails==NULL && strstr(filename,"MESSAGES")) { // FIXME This is an UGLY HACK
		mails=fopen(filename,"a+");
		fwrite(_(welcome),strlen(_(welcome)),1,mails);
		fseek(mails,0L,SEEK_SET);
	}
	else if (mails==NULL) {
		gchar *msg=new gchar[strlen(filename) + 17];
		sprintf(msg,_("File not found: %s"),filename);
		gnome_error_dialog_parented( msg, GTK_WINDOW(app));
		return 0;
	}
	flock(fileno(mails), LOCK_SH);

	gchar* line=new gchar[201];

	gtk_clist_freeze(mail_list);

	gboolean head=TRUE;
	while (!feof(mails)) {
		fgets(line,200,mails);
		if (*line=='\n') head=FALSE;
		else if (memcmp(line,"From ",5)==0) { 
			num_msg++;
			gtk_clist_append(mail_list,empty);
			FileMsgData* msg_data = new FileMsgData;
			msg_data->filename = filename;
			msg_data->filepos = ftell(mails)-strlen(line);
			msg_data->row = num_msg;
			CMessage *msg = new CMessage(this,msg_data);
			gtk_clist_set_row_data_full(mail_list,num_msg,msg,free_data);
			head=TRUE;
		// The Row Data is now set to a pointer to the file position
		// of the E-Mail in this row.
		// With a simple fseek() we can then jump to it later on.
		}
		else if (head && memcmp(line,"From:",5)==0) gtk_clist_set_text(mail_list,num_msg,2,decode_subj(line+6));
		else if (head && memcmp(line,"Subject:",8)==0) gtk_clist_set_text(mail_list,num_msg,0,decode_subj(line+9));
		else if (head && memcmp(line,"Date:",5)==0) gtk_clist_set_text(mail_list,num_msg,1,(line+6));
		else if (head && memcmp(line,"X-MMC:",6)==0) {
			int flags=atoi(line+7);
			if (!(flags & FLAG_READ)) {
				gtk_clist_set_foreground(mail_list,num_msg,
					COL_UNREAD_F);
				unread++;
			}
		}

	}
	flock(fileno(mails), LOCK_UN);
	fclose(mails);
	gtk_clist_thaw(mail_list);
	delete[] line;

	gchar lbl[200];
	snprintf(lbl,200,_("%li Messages in the folder (%li unread)"),num_msg+1,
		unread);
	gtk_object_set_data(GTK_OBJECT(mail_list), "unread", GINT_TO_POINTER(unread));

	gnome_appbar_push(GNOME_APPBAR(status), lbl);
	return num_msg+1;
}

int CFolderFile::select(guint row) {
	void* data = gtk_clist_get_row_data(mail_list, row);
	return select(data);
}

int CFolderFile::select(void* data) {
	FileMsgData *msg_data = (FileMsgData*)data;
	gchar *raw = get(data);
	CFolder::select(raw, fl, isqp, msg_data->row, data);
	delete[] raw;
	return msg_data->row;
}

gchar* CFolderFile::get(void* data) {
	FileMsgData *msg_data = (FileMsgData*)data;
	FILE *mails;
	gchar *s;
	GString* mail=g_string_new("");
	gchar line[201];
	mails = fopen(msg_data->filename, "r");
	if (!mails) return NULL;
	
	fseek(mails,msg_data->filepos, SEEK_SET);
	fgets(line,200,mails); // to skip the first "From " line
	do {
		fgets(line,200,mails);
		if (strncmp(line,"From ",5) == 0) break;
		g_string_append(mail,line);
	} while (!feof(mails));
	fclose(mails);
	s=mail->str;
	g_string_free(mail,FALSE);
	return s;
	
}

int CFolderFile::addMail(const gchar *msg) {
	const gchar* delim = "From aaa@bbb.ccc Mon Oct 30 13:46:15 2000\n";
	FILE *folder = fopen(filename,"a");
	if (!folder) g_warning("Open of %s for appending failed!", filename);
	flock(fileno(folder), LOCK_EX);
	fwrite(delim,strlen(delim),1,folder);
	fwrite(msg,strlen(msg),1,folder);
	fwrite("\n",1,1,folder);
	flock(fileno(folder), LOCK_UN);
	fclose(folder);
	return 0;
	
}

gchar* CFolderFile::get(gint row) {
	void* data = gtk_clist_get_row_data(mail_list, row);
	return get(data);
}

int CFolderFile::del(gint row) {
	CMessage *msg = (CMessage*)gtk_clist_get_row_data(mail_list,row);
	if (msg) { msg->del(); return 0; }
	else return -1;
}

int CFolderFile::del(void* data) {
	FileMsgData *msg_data = (FileMsgData*)data;
/*	if (!(mail_list->selection)) {
		gnome_app_error(GNOME_APP(app), _("Please choose a message to delete!"));
		return -1;
	}*/

	gchar *fromname=msg_data->filename;
	gchar *dir = dup(fromname);
	gchar* tmp = strrchr(dir, PATH_SEP);
	if (tmp) *tmp = 0;
	gchar *toname=g_concat_dir_and_file(dir, "mmc.XXXXXX");

	FILE* from;
	gint to = mkstemp(toname);
	if (to == -1) {
		switch (errno) {
		case EEXIST:
			gnome_app_error(GNOME_APP(app), _("Could not make a unique filename."));
			break;
		case EINVAL:
			gnome_app_error(GNOME_APP(app), _("Invalid mkstemp() template. Bug detected.\n"
				"Contact the author."));
			break;
		}
	}

	delete[] dir;

	gchar line[100];
	gint len=0;

	gint ans;

	from=fopen(fromname,"r");
	flock(fileno(from), LOCK_SH);

	while (ftell(from) < msg_data->filepos) {
		if (feof(from)) return -1; // Should not occur anyway.
		fgets(line,100,from);
		len=strlen(line);
		write(to, line, len);
	}
	fgets(line,100,from); // FIXME/TODO wtf is this good for?
	fgets(line,100,from);
	fgets(line,100,from);

	while (memcmp(line,"From ",5) != 0) {
		fgets(line,100,from);
		len=strlen(line);
		if (feof(from)) break;
	}

	while (!feof(from)) {
		write(to, line, len);
		fgets(line,100,from);
		len=strlen(line);
	}
	
	flock(fileno(from), LOCK_UN);
	fclose(from);
	close(to);

	ans=rename(toname, fromname);
	g_free(toname);

	parse();

	return 0;
}

void CFolderFile::markRead(void* data) {
	FileMsgData *msgData = (FileMsgData*)data;
	FILE* mails = fopen(msgData->filename,"r+");
	flock(fileno(mails), LOCK_EX);
	gchar buf[100];
	if (mails == NULL) return;

	fseek(mails, msgData->filepos, SEEK_SET);
	fgets(buf, 100, mails); // To skip the initial "From " Line.
	do {
		fgets(buf, 100, mails);
		if (strncasecmp(buf,"X-MMC:",6)==0) {
			fseek(mails, (-1)*strlen(buf), SEEK_CUR);
			gchar *wr="X-MMC: 1\n"; // TODO: This implies that "Read" is the only possible flag...
			fwrite(wr,strlen(wr),1,mails);
			break;
			
		}
	} while (!feof(mails) && strncasecmp(buf,"From ",5));

	flock(fileno(mails), LOCK_UN);
	fclose(mails);
}

gchar *getName(gchar *name) {
	gchar *fname;
	if (!name) {
		return NULL;
		g_warning(_("Passed NULL to getName - probably Bug!"));
	}

	if (memcmp(name,"{MD}",4)==0) {
		fname=new gchar[strlen(name)-4+strlen(g_mmcHome)+1];
		sprintf(fname,"%s%s",g_mmcHome,name+4);
	}
	else if (*name == '~') {
		fname = new gchar[strlen(name)-1+strlen(getenv("HOME"))+1];
		sprintf(fname,"%s%s",getenv("HOME"), name+1);
	}
	else fname=dup(name);
			

	return fname;
}

void select_box(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
	gtk_clist_clear(GTK_CLIST(mail_list));
	gtk_editable_delete_text(GTK_EDITABLE(view_mail), 0, -1);
	((CFolder*)gtk_clist_get_row_data(GTK_CLIST(widget), row))->parse();
}

void do_sort(GtkCList *widget, guint col, gpointer data) {
	GtkSortType sorting=GTK_SORT_ASCENDING;
	if (col==sort_col) ascending=~ascending;
	else sort_col=col;
	
	switch (ascending) {
		case TRUE:
			sorting=GTK_SORT_ASCENDING;
			break;
		case FALSE:
			sorting=GTK_SORT_DESCENDING;
			break;
	}

	gtk_clist_set_sort_type(widget, sorting);

	gtk_clist_set_sort_column(widget, col);

	if (col == DATECOL) gtk_clist_set_compare_func(widget,
		datesort);
	
	gtk_clist_sort(widget);
}

gint datesort(GtkCList *list, gconstpointer date1, gconstpointer date2) {
return 0;
}

void doSearch(GtkWidget *foo, gpointer data) {
	GtkWidget **widgets=(GtkWidget**)data;
	gchar *find = gtk_entry_get_text(GTK_ENTRY(widgets[1]));

	g_free(find);	
	gtk_widget_destroy(widgets[0]);
	delete[] widgets;
}

void search(GtkWidget* foo, gpointer bar) {
	GtkWidget *sDialog; // Search Dialog
	GtkWidget *box, *fixed;
	GtkWidget *searchText = gnome_entry_new("searchfor");
	
	GtkWidget **widgets = new GtkWidget*[2];
	sDialog = gnome_dialog_new(_("Search"), _("Search"), 
		GNOME_STOCK_BUTTON_CANCEL, NULL);
	box = GNOME_DIALOG(sDialog)->vbox;
	fixed = gtk_fixed_new();
	
	gtk_fixed_put(GTK_FIXED(fixed), gtk_label_new("Search for:"), 8, 8);
	gtk_fixed_put(GTK_FIXED(fixed), searchText, 80, 8);
	
	widgets[0]=sDialog;
	widgets[1]=gnome_entry_gtk_entry(GNOME_ENTRY(searchText));
	
	gnome_dialog_button_connect(GNOME_DIALOG(sDialog), 0, 
		GTK_SIGNAL_FUNC(doSearch), widgets);
	gnome_dialog_button_connect(GNOME_DIALOG(sDialog), 1, 
		GTK_SIGNAL_FUNC(destroy), sDialog);
	
	gtk_box_pack_start(GTK_BOX(box), fixed, TRUE, TRUE, 0);
	gtk_widget_show_all(sDialog);
}

/*
   char *ucase(char *string);

   Converts the given string to uppercase.

   Returns: a copy of the string, all letters are uppercase. Should be g_freed later
*/
char *ucase(char *string) {
	char *ptr=dup(string);
	char *vptr=ptr;

	while (*ptr!=0) *ptr++=toupper(*ptr);
	return vptr;
}

TRANSFER_ENCODING getencoding (char *up) {
	if (memcmp((up+27),"QUOTED-PRINTABLE",16)==0) return QUOTED_PRINTABLE;
	else if (memcmp(up+27,"8BIT",4)==0) return BIT8;
	else if (memcmp(up+27,"7BIT",4)==0) return BIT7;
	else if (memcmp(up+27,"BASE64",6)==0) return BASE64;
	else { g_print(_("We don't yet support encoding %s"),up+27); return ENC_NOTSUPP; }
}

CONTENT_TYPE gettype(char *up) {
	up+=14;
	while ((*up == ' ') || (*up=='\t')) up++;
	if (memcmp((up),"TEXT/PLAIN",10)==0) return TEXT_PLAIN;
	else if (memcmp((up),"TEXT/HTML",9)==0) return TEXT_HTML;
	else if (memcmp((up),"MULTIPART/MIXED",15)==0) return MULTIPART_MIXED; // Has (probably) attachments
	else if (memcmp((up),"MULTIPART/ALTERNATIVE",21)==0) return MULTIPART_ALTERNATIVE;
	else if (memcmp((up),"APPLICATION/OCTET-STREAM",23)==0) return OCTET_STREAM;
	else if (memcmp((up),"APPLICATION/ZIP",14)==0) return APP_ZIP;
	else {
		//g_print(_("Content-Type not supported: %s\n"),up);
		return TYPE_NOTSUPP;
	}
}

void select_mail(GtkWidget *widget, gint row, gint column, GdkEventButton event, gpointer data) {
	gtk_editable_delete_text(GTK_EDITABLE(view_mail), 0, -1);
	attach.reset();
	if (GTK_IS_WIDGET(attach_window)) {
		gnome_dialog_close(GNOME_DIALOG(attach_window));
		attach_window=NULL;
	}
	CMessage *msg=(CMessage*)gtk_clist_get_row_data(GTK_CLIST(widget),row);
	if (!msg) return;
	msg->select();
}

void show_all_head(GtkWidget* widget, gpointer data) {
	GdkEventButton ev;
	fl=!fl;
	select_mail(mail_list, get_row(GTK_CLIST(mail_list)), 0, ev, NULL);
}

void is_qp(GtkWidget * widget, gpointer data) {
	GdkEventButton ev;
	isqp=!isqp;
	select_mail(mail_list, get_row(GTK_CLIST(mail_list)), 0, ev, NULL);
}

void doCopy(GtkWidget *widget, gpointer data) {
	GtkWidget **widgets = (GtkWidget**)data;
	gint row;
	if (GTK_CLIST(widgets[0])->selection)
		row=GPOINTER_TO_INT(GTK_CLIST(widgets[0])->selection->data);
	else {
		gnome_warning_dialog_parented(_("Please choose a folder from the List!"), GTK_WINDOW(widgets[1]));
		return;
	}
	CFolder *dest = (CFolder*)gtk_clist_get_row_data(GTK_CLIST(widgets[0]), row);

	gtk_widget_destroy(widgets[1]);
	
	gnome_appbar_push(GNOME_APPBAR(status), _("Copying Message..."));
	while (gtk_main_iteration_do(FALSE));

	gint srcRow = GPOINTER_TO_INT(GTK_CLIST(mail_list)->selection->data);
	
	CMessage *srcMsg = (CMessage*)gtk_clist_get_row_data(GTK_CLIST(mail_list), srcRow);
	gchar *msg = srcMsg->getRaw();
	dest->addMail(msg);
	delete[] msg;

	gnome_appbar_pop(GNOME_APPBAR(status));


	if (widgets[2]) del(NULL,NULL);
	
}

void copy(GtkWidget *foo, gpointer move) {
	guint row;
	if (GTK_CLIST(mail_list)->selection) {
		row=GPOINTER_TO_INT(GTK_CLIST(mail_list)->selection->data);
	}
	else {
		gnome_app_warning(GNOME_APP(app), _("Please choose a Message to copy or move!"));
		return;
	}

	if ((GTK_CLIST(mail_list)->rows) == 0) {
		gnome_app_warning(GNOME_APP(app), _("Nothing to copy/move!"));
		return;
	}

	//we should ask for the destination folder
	GtkWidget *wnd, *box, *bbox;
	GtkWidget *list;
	GtkWidget *ok, *cancel;

	GtkWidget **widgets = new GtkWidget*[3];

	wnd=gtk_window_new(GTK_WINDOW_DIALOG);
	box=gtk_vbox_new(FALSE,0);
	bbox=gtk_hbox_new(TRUE,10);

	list = g_folders.makeCList();

	gtk_clist_set_column_title(GTK_CLIST(list), 0, _("Please select a destination folder:"));
	gtk_clist_column_titles_show(GTK_CLIST(list));
#if 0
	list=gtk_clist_new(1);

	for (int i=0; i< GTK_CLIST(mboxes)->rows; i++) {
		char *content;
		gpointer data;
		gtk_clist_get_text(GTK_CLIST(mboxes), i, 0, &content);
		data=gtk_clist_get_row_data(GTK_CLIST(mboxes), i);
		gtk_clist_append(GTK_CLIST(list), &content);
		gtk_clist_set_row_data(GTK_CLIST(list), i, data);
	}
#endif
	widgets[0]=list;
	widgets[1]=wnd;
	widgets[2]=(GtkWidget*)move;
		
	ok=gtk_button_new_with_label(_("OK"));
	cancel=gtk_button_new_with_label(_("Cancel"));

	gtk_box_pack_start(GTK_BOX(bbox), ok, TRUE, TRUE, 10);
	gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 10);

	gtk_box_pack_start(GTK_BOX(box), list, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(box), bbox, FALSE, FALSE, 0);

	gtk_signal_connect(GTK_OBJECT(cancel), "clicked", GTK_SIGNAL_FUNC(destroy),
			wnd);
	gtk_signal_connect(GTK_OBJECT(ok), "clicked", GTK_SIGNAL_FUNC(doCopy),
			widgets);


	gtk_container_add(GTK_CONTAINER(wnd), box);

	gtk_widget_show_all(wnd);
}

void move(GtkWidget *foo, gpointer bar) {
	copy(NULL,GINT_TO_POINTER(1));
}

void del(GtkWidget *foo, gpointer bar) {
	if (!(GTK_CLIST(mail_list)->selection)) {
		gnome_app_error(GNOME_APP(app), _("Please choose a message to delete!"));
		return;
	}
	CMessage *msg = (CMessage*)gtk_clist_get_row_data(GTK_CLIST(mail_list),get_row(GTK_CLIST(mail_list)));
	if (msg) msg->del();
}

void free_data(gpointer data) {
	CMessage *par=(CMessage*)data;

	delete par;
}

void close_dialog(GtkWidget *foo, gpointer data) {
	GtkWidget **widget = (GtkWidget**)data;
	g_assert(widget != NULL);
	g_assert(*widget != NULL);
	gnome_dialog_close(GNOME_DIALOG(*widget));
	*widget=NULL;
}

void attachm(GtkWidget *widget, gpointer data) {
	// Display a list of the attachments
	GtkWidget *attachlist;
	GtkWidget *attachbox;
	GtkWidget *fixed;

	gchar **name=attach.list();
	gchar **type=attach.types();
	gchar **enc=attach.encodings();
		
	if (*type==NULL) {
		gnome_app_warning(GNOME_APP(app),_("Message has no attachments!"));
		return;
	}
	
	attach_window=gnome_dialog_new(_("Choose Attachment"),
		_("Save"), GNOME_STOCK_BUTTON_CANCEL, NULL);
	gnome_dialog_set_parent(GNOME_DIALOG(attach_window), GTK_WINDOW(app));
	attachbox=GNOME_DIALOG(attach_window)->vbox;

	attachlist=gtk_clist_new(3);
	gtk_clist_set_column_title(GTK_CLIST(attachlist),0,_("Filename"));
	gtk_clist_set_column_title(GTK_CLIST(attachlist),1,_("Content-Type"));
	gtk_clist_set_column_title(GTK_CLIST(attachlist),2,_("Encoding"));

	gtk_clist_column_titles_show(GTK_CLIST(attachlist));
	gtk_clist_column_titles_passive(GTK_CLIST(attachlist));

	gchar *add[4]={ NULL, NULL, NULL, NULL };

	for (; *type!=NULL; name++,type++,enc++) {
		add[0]=*name;
		add[1]=*type;
		add[2]=*enc;
		gtk_clist_append(GTK_CLIST(attachlist), add);
	}

	for (guint col=0; col<3; col++) {
		guint width= gtk_clist_optimal_column_width(GTK_CLIST(attachlist),col);
		gtk_clist_set_column_width(GTK_CLIST(attachlist), col, width);
	}
	
	file_entry=gnome_file_entry_new(NULL, _("Save Attachment as"));
	gnome_file_entry_set_default_path(GNOME_FILE_ENTRY(file_entry), getenv("HOME"));

	gnome_dialog_button_connect(GNOME_DIALOG(attach_window), 0, GTK_SIGNAL_FUNC(save_att), attachlist);
	gnome_dialog_button_connect(GNOME_DIALOG(attach_window), 1, GTK_SIGNAL_FUNC(close_dialog), &attach_window);

	fixed=gtk_fixed_new();
	gtk_fixed_put(GTK_FIXED(fixed), gtk_label_new(_("Filename:")), 8, 8);
	gtk_fixed_put(GTK_FIXED(fixed), file_entry, 80, 8);

	gtk_box_pack_start(GTK_BOX(attachbox), attachlist, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(attachbox), fixed, FALSE, FALSE, 0);
	
	gtk_widget_show_all(attach_window);
}

void save_att(GtkWidget *widget, gpointer data) {

	GtkCList *attach_list=GTK_CLIST(data);
	guint row;
	gchar *decoded_att;
	gchar *enc;

	gchar *filename=gnome_file_entry_get_full_path(
		GNOME_FILE_ENTRY(file_entry),FALSE);
	if (!filename || !*filename) {
		gnome_warning_dialog_parented(_("Please enter a filename!"), GTK_WINDOW(attach_window));
		return;
	}
	FILE *output;
	guint len;
	
	if (attach_list->selection) {
		row=GPOINTER_TO_INT(attach_list->selection->data);
	}
	else {
		gnome_warning_dialog_parented(_("Please choose an attachment from the list!"), GTK_WINDOW(attach_window));
		return;
	}

	gtk_clist_get_text(GTK_CLIST(attach_list), row, 2, &enc);

	// Get the attachment
	decoded_att=attach.get(row, &len, enc);

	// Save the attachment
	output=fopen(filename,"w");
	if (!output) {
		gnome_error_dialog_parented(_("Error opening output file!"), GTK_WINDOW(attach_window));
		return;
	}

	fwrite(decoded_att, len, 1, output);

	fclose(output);
	g_free(filename);

	gnome_ok_dialog_parented(_("Attachment successfully saved."), GTK_WINDOW(attach_window));
}

ATTACHMENTS::ATTACHMENTS() {
	for (guint i=0; i<=count; i++) {
		for (guint x=0; x<4; x++) {
			data[i][x]=NULL;
		}
		offset[i]=0;
		fdata[i]=NULL;
	}
	folder=NULL;
}

/*
ATTACHMENTS:add(gchar *fname, guint pos, gchar *real_fn)

Adds an attachment to the list.

Parameters:
folder:  A CFolder to access the message
data:    The data folder needs to get the attachment
offset:  The distance to the beginning of the message
pos:     The position of the file where the first Base64 Letter is
real_fn: The Name of the attachment
*/
void ATTACHMENTS::add(CFolder *fold, void* dat, guint off, const gchar *real, const gchar *type, TRANSFER_ENCODING enc, gchar *bound) {
	gchar *e="";

	if (real==NULL) real="";

	switch (enc) {
	case BIT8:
		e="8bit";
		break;
	case BIT7:
		e="7bit";
		break;
	case QUOTED_PRINTABLE:
		e="quoted-printable";
		break;
	case BASE64:
		e="base64";
		break;
	case ENC_NOTSUPP:
		e="unsupported";
		break;
	}

	if (*real=='\"') real++;
	gchar *fn=g_strdup(real);
	if (gchar *x=strstr(fn,"\"")) *x=0;

	if (fold) folder=fold;
	fdata[count]=dat;
	offset[count]=off;
	
	data[count][0]=decode_subj(fn);
	data[count][1]=g_strdup(type);
	data[count][2]=g_strdup(e);
	data[count][3]=g_strdup(bound);

	count++;
}

gchar** ATTACHMENTS::getdata(guint which) {
	gchar **ret=new gchar*[count+1];

	for (guint i=0; i<count; i++)
		ret[i]=data[i][which];

	ret[count]=NULL;

	return ret;
}

/*
  gchar  **list();

Returns: An array of strings.
         Each string is the filename of an attachment.
*/
gchar** ATTACHMENTS::list() {
	return getdata(0);
}

gchar** ATTACHMENTS::types() {
	return getdata(1);
}

gchar** ATTACHMENTS::encodings() {
	return getdata(2);
}

void ATTACHMENTS::reset() {
	for (guint i=0; i<=count; i++) {
		for (guint x=0; x<4; x++) {
			if (data[i][x]) g_free(data[i][x]);
			data[i][x]=NULL;
		}
		fdata[i]=NULL;
		offset[i]=0;
	}

	count=0;
}

gchar *ATTACHMENTS::get(guint which, guint *len, gchar *enc) {
	gulong size=0;
	if (which>count) return NULL;
	gchar *decoded;

	gchar *bound=data[which][3];

	gchar *buffer;

	buffer = folder->get(data[which]);
	if (strstr(buffer,bound)) *(strstr(buffer,bound))=0;
	
	if (strcasecmp(enc,"base64")==0) decoded = decode_b64(buffer, len);
	else if (strcasecmp(enc,"quoted-printable")==0) {
		decoded = strdup(decode_mime(buffer));
		*len = size;
	}
	else {
		*len=size;
		decoded=new gchar[size];
		memcpy(decoded, buffer, size);
	}

	delete[] buffer;

	return decoded;
}
