/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/

#ifndef __MMC_MSG_H
#define __MMC_MSG_H

#include <gtk/gtk.h>
#include <glib.h>
#include <stdio.h>
#include "protocols.h"

void select_mail(GtkWidget*,gint,gint,GdkEventButton,gpointer); 
void show_all_head(GtkWidget* widget, gpointer data);
void is_qp(GtkWidget*,gpointer);
char *ucase(char *line);
void attachm(GtkWidget *,gpointer);
void del(GtkWidget *,gpointer);
void save_att(GtkWidget*,gpointer);
void do_sort(GtkCList*,guint,gpointer);
void select_box(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data);
void copy(GtkWidget *widget, gpointer data);
void move(GtkWidget*,gpointer);
void search(GtkWidget*,gpointer);
void close_dialog(GtkWidget*,gpointer);
gchar* getName(gchar*);
int get_row(GtkCList *list);

void free_data(gpointer data);

class CMessage {
public:
	// --- [ Con- and destructors ] ---
	CMessage();
	CMessage(CFolder*, gpointer);
	//used by CFolder.
	CMessage(gchar *from,  gchar *replyto,  gchar *to,
		 gchar *cc,  gchar *bcc,  gchar *subject,
		 gchar *additional,  gchar *body);
	//only useful when used with compose();
	
	~CMessage();

	// --- [ Composing Functions ] --- (see compose.cc)
	void compose();
	// composes a message. depending on the used constructor, either
	// empty or with the fields filled out.
	void reply();
	void forward();
	// function is obvious.
	
	// --- [ Get the message ] ---
	gchar *getBody(); // \ 
	gchar *getHead(); // |- delete[] result;
	gchar *getRaw(); //  /
	
	void select();
	// displays the message
	
	// ---[ Modify the Message ] ---
	void del();

	void markRead() {
		folder->markRead(data);
	};
protected:
	void fill(); // fills from etc. with the correct values

	CFolder *folder;
	void *data;
	
	gchar *from;
	gchar *to;
	gchar *cc;
	gchar *bcc;
	gchar *subj;
	gchar *date;
	gchar *replyto;
	GString *additional;
	gchar *body;
	gchar *msgid;
};

enum CONTENT_TYPE { // Content-Types
	TEXT_PLAIN,
	TEXT_HTML,
	MULTIPART_MIXED,
	MULTIPART_ALTERNATIVE,
	OCTET_STREAM,
	APP_ZIP,
	TYPE_NOTSUPP
};

enum TRANSFER_ENCODING { // Content-Transfer-Encondings
	BIT7,
	BIT8,
	QUOTED_PRINTABLE,
	BASE64,
	ENC_NOTSUPP
};

TRANSFER_ENCODING getencoding (char *up);
CONTENT_TYPE gettype(char *up);

struct FileMsgData {
	gchar *filename;
	long filepos;
	gint row;
};

class CFolderFile : public CFolder {
	public:
		CFolderFile();
		~CFolderFile();
		CFolderFile(gchar *fname, gchar *savedname, gchar *desc, GtkCList* list, GtkText *disp);
		virtual char *query() const {
			return "File (mbox)";
		}
		virtual void free(void* data) { delete (FileMsgData*)data; }
		virtual int init(GtkCList *mails, GtkText *text);
		virtual int parse();
		virtual int select(guint row);
		virtual int select(void* data);

		virtual int create();
		virtual int addFolder();
		virtual int remove(gboolean onlyfromlist);

		void doCreate( gchar *string, gpointer data);
		void doAdd(gchar *string, gchar *fname);

		virtual int addMail(const gchar *text);
		virtual char* get(gint row);
		virtual char* get(void* data);
		virtual int del(gint row);
		virtual int del(void* data);
		virtual void markRead(void*);
	protected:
		gchar *filename;
		gchar *description;
		gchar *saved_name;
};

class ATTACHMENTS {
	public:
		ATTACHMENTS();
		void reset();
		gchar *get(guint which, guint *len, gchar *enc);
		void add(CFolder* folder, void* data, guint offset, const gchar *filename, const gchar *type, TRANSFER_ENCODING enc,
				gchar *boundary);
		// Explanatation for add:
		// Folder is the CFolder for accessing the mail, data the CFolder's data to find the mail
		// offset is the distance to the beginning of the message (in gchars = usually bytes).

		gchar** list();
		gchar** types();
		gchar** encodings();
		
	protected:
		gchar** getdata(guint which);
		guint offset[20];  //   We assume that there will be only 20 attachments - maybe TODO
		void *fdata[20];
		CFolder *folder;

		gchar *data[20][4];
		// Format for data: [0] Filename
		//                  [1] MIME-Type
		//                  [2] Encoding
		//                  [3] Boundary

		guint count;
};

#define FLAG_READ 1

#endif
