/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/
/* Note: The functions recvline and recvch are (c) 1999 Susixware
 * I took them from the source of their Program Spruce, version 0.5.15.
 * Spruce available at: http://www.xtorshun.org/spruce/ or
 * http://spruce.sourceforge.net
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <pthread.h>
#include "pop3.h"
#include "main.h"
#include "msg.h"
#include "progress.h"

struct get_param {
	guint32 ip;
	gint port;
	//gchar *file;
	CFolder* folder;
	gchar *user;
	
};

typProgressData *pdata = NULL;

void get_mail(GtkWidget *widget, gpointer data) {
	gchar *protname, *idstring;
	gint len;
	gchar *accountname = gnome_config_get_string("/mmc/accounts/current");

	if (*accountname == 0) {
		gnome_app_error(GNOME_APP(app),
			_("Please configure MMC before attempting to get mails!"));
	}
	
	gchar *qry_string = new gchar[31];
	snprintf(qry_string,30,"/mmc/accounts/%s",accountname);

	gchar *cnf_string = gnome_config_get_string(qry_string);

	gchar *ptr = strstr(cnf_string,"@@");
	len = ptr-cnf_string;

	protname = new gchar[len+1];
	protname[len]=0;
	if (ptr) strncpy(protname,cnf_string,len);
	else {
		gnome_app_error(GNOME_APP(app),
		_("Could not parse Account Configuration String"
		"Maybe you have not configured MMC?"));
		return;
	}

	ptr += 2;
	gchar *ptr2 = ptr;
	ptr = strstr(ptr2,"@@");
	len = ptr - ptr2;

	idstring = new gchar[len+1];
	idstring[len]=0;
	strncpy(idstring, ptr2, len);	

	CProto *protocol = g_pHandler.get(protname);
	g_assert(protocol != NULL);

	protocol->get(idstring);

	g_free(accountname);
	g_free(qry_string);
	g_free(cnf_string);
	delete[] protname;
	delete[] idstring;
	
}

#if 0
void get_mail(GtkWidget *widget, gpointer data) {
/*int  pthread_create(pthread_t  *  thread, pthread_attr_t *
       attr, void * (*start_routine)(void *), void * arg);*/
	pthread_t *thread=new pthread_t;
	if (pthread_create(thread, NULL, fetchmail, NULL))
		gnome_error_dialog_parented(
			_("Error retrieveing Mails\n"
			"Couldn't spawn thread."),
			GTK_WINDOW(app));

}
#endif

void *get_thread(void* arg);

void gotpw(gchar *pass, gpointer dat) {
	gpointer* data = (gpointer*)dat;
	CProtoPop3 *proto = (CProtoPop3*)data[0];
	proto->retr(pass, data[1]);
	delete[] data;

	while (gtk_main_iteration_do(TRUE));
}

void found_host(guint32 ip, gpointer data) {
	CProtoPop3 *folder = (CProtoPop3*)data;
	gchar *user, *pass, *sport;
	gint port;
	
	user = gnome_config_get_string("/mmc/pop3/user=");
	pass = gnome_config_private_get_string("/mmc/pop3/pass=");
	sport = gnome_config_get_string("/mmc/pop3/port=110");

	port = atoi(sport);

	if ( *user == 0) {
		gnome_app_warning(GNOME_APP(app), _("No Username defined!"));
		return;
	}
	else if ((port <= 0) || (port >= 65535)) {
		gnome_app_warning(GNOME_APP(app), _("Invalid Port!"));
	}

	get_param *par=new get_param;

	par->user=user;
	par->folder=g_folders.get("INBOX");
	par->port=port;
	par->ip=ip;


	g_free(sport);


	if (*pass == 0) {
		g_free(pass);
		gchar *server = gnome_config_get_string("/mmc/pop3/server=");
		gchar *prompt = new gchar[201];
		snprintf(prompt, 200, "Please enter your password for user %s at server %s!", user, server);
		g_free(server);

		gpointer* dat = new gpointer[2];
		dat[0] = folder;
		dat[1] = par;
		
		gnome_app_request_password(GNOME_APP(app),
			prompt, GTK_SIGNAL_FUNC(gotpw), dat);
	}
	else {
		folder->retr(pass, par);
		g_free(pass);
	}

/*	if (retr(ip,110,GetFileName(1),user,pass,(widgets*)data)==OK) {
		select_box(mboxes,0,0,NULL,mboxes);
	}
*/

/*	pthread_t thread;
	if (pthread_create(&thread,NULL, get_thread, par)) {
		gnome_error_dialog_parented(
			_("Error retrieving mails:\n"
			"Couldn't spawn thread."), GTK_WINDOW(app));
	}
	
*/ // Threads don't work well enough for them to be used			
}

void *get_thread(void* arg) {
	get_param *par=(get_param*)arg;
/*	if (retr(par->ip,par->port,par->file,par->user,par->pass,NULL)==OK)
		select_box(mboxes,0,0,NULL,mboxes);
*/
	g_free(par->user);
	delete par;
	return NULL;
}

gboolean seek_in_file(const gchar *filename, const gchar *string) {

	gchar *line=new gchar[100];
	gboolean c=FALSE;
	FILE *file=fopen(filename,"r");
	if (file==NULL) return FALSE;

	while (!feof(file)) {
		fgets(line,99,file);
		if (strstr(line,string)) { c=TRUE; break; }
	}
	fclose(file);
	delete[] line;
	return c;
}

char* _itoa(int number, char *buffer) {

	sprintf(buffer,"%i",number);
	return buffer;

	// could use snprintf(char* string, int MaxCharacters, char* format, ...);

}



int conn(const char *hostname, int port) {
	int socknum;
	struct hostent *hostInfo;
	struct sockaddr_in sin;

	if (isdigit(*hostname))
		sin.sin_addr.s_addr=inet_addr(hostname);
	else if ((hostInfo = gethostbyname(hostname)))
		memcpy((char*)&sin.sin_addr, hostInfo->h_addr, hostInfo->h_length);
	else
		return 0;

	bzero(&(sin.sin_zero), 8);
	sin.sin_port=htons(port);
	sin.sin_family=AF_INET;
  
	socknum = socket(AF_INET, SOCK_STREAM, 0);
 
	if (socknum < 0) return 0;
	if (connect(socknum, (struct sockaddr*)&sin, sizeof(sin)) < 0) return 0;
	return socknum;
}

int conn(guint32 ip, int port) {
	int socknum, ret;
	struct sockaddr_in sin;

	sin.sin_addr.s_addr=htonl(ip);
	bzero(&(sin.sin_zero), 8);
	sin.sin_port=htons(port);
	sin.sin_family=AF_INET;

	socknum=socket(AF_INET, SOCK_STREAM, 0);

	if (socknum<0) return 0;

	ret=connect(socknum, (struct sockaddr*)&sin, sizeof(sin));
	if (ret<0) return 0;

	return socknum;
}

int CProtoPop3::get(const gchar* data) {
/*
	// Parse data string
	// it's format is:
	// server:port:username:password
	// so don't use : in username/password
	
	gchar *ptr;
	
	gchar *server = oneLine(data, ":");
	data += strlen(server);
	if ((ptr = strstr(server, ":"))) *ptr=0;
	
	gchar *portS = oneLine(data, ":");
	if ((ptr = strstr(portS, ":"))) *ptr=0;
	guint32 port = atol(portS);
	data += strlen(portS);
	delete[] portS;

	gchar *user = oneLine(data, ":");
	if ((ptr = strstr(user, ":"))) *ptr=0;
	data += strlen(user);

	gchar *pass = oneLine(data, ":");
	if ((ptr = strstr(pass, ":"))) *ptr=0;
*/
	gchar *server;
	server = gnome_config_get_string("/mmc/pop3/server=");
	if (*server==0) {
		gnome_error_dialog_parented(
			_("Error retrieving mails. No Servername given."),
			GTK_WINDOW(app));
		return -1;
	}

	StartProgress();
	UpdateLabel(_("Connecting"));
	UpdateProgress(0,100);


	gnome_dns_lookup(server, &found_host, this);
	g_free(server);

	return 0;

	return 0;
}

PROTO_ERR CProtoPop3::retr(const gchar *pass, gpointer data) {
	get_param *par = (get_param*)data;
	guint32 ip = par->ip;
	gint port = par->port;
	CFolder* folder = par->folder;
	gchar *user = par->user;

	delete par;


	char result[10000];
	int err;
	long numMsg;
	char command[100];
	long total_size;

	char lbl[200];

	if (port==0 || folder == NULL || pass==0 || *user==0 || *pass==0) {
		return ERR_CONN;
	}
	
	//ok, start retrieving the msgs
	
	while (gtk_main_iteration_do(TRUE));
	
	char *uidfile=""; //GetFileName(0);

	int socknum=conn(ip,port);

	if (socknum==0) {
		error(FLAG_RECV, _("Connect"), NULL);
		return ERR_CONN;
	}

	UpdateLabel(_("Authenticating: Sending USER command..."));

	recvline(socknum,result,9999);

	strcpy(command,"USER ");
	strcat(command,user);
	strcat(command,"\r\n");

	send(socknum,command,strlen(command),0);

	err=recvline(socknum,result,9999);

	if ( *result != '+' || err < 0) {
		error(FLAG_RECV, "USER", result);
		close(socknum);
		return ERR_AUTH;
	}

	UpdateLabel(_("Authenticating: Sending PASS command..."));
	
	strcpy(command,"PASS ");
	strcat(command,pass);
	strcat(command,"\r\n");
	
	send(socknum,command,strlen(command),0);

	err=recvline(socknum,result,9999);

	if ( *result != '+' || err < 0) {
		error(FLAG_RECV, "PASS",result);
		close(socknum);
		return ERR_AUTH;
	}

	send(socknum,"STAT\r\n",6,0);
	
	err=recvline(socknum,result,9999);

	if( *result != '+' || err < 0) {
		error(FLAG_RECV, "STAT", result);
		close(socknum);
		return ERR_OTHER;
	}

	char *begin=(strstr(result," ")+1);
	char *ptr=strstr(begin," ")+1;

	total_size=atol(ptr);

	*(ptr-1)=0;

	numMsg=atol(begin); // from here on we have got the number of messages and can retrieve them.

	snprintf(lbl,100,"%li Messages on the server",numMsg);
	UpdateLabel(lbl);

	UpdateProgress(0, numMsg);

	for (int i=1; i<=numMsg; i++) {
		char msg_num[10];
		_itoa(i,msg_num);
		strcpy(command,"LIST ");
		strcat(command,msg_num);
		strcat(command,"\r\n");

		send(socknum,command,strlen(command),0);
		err=recvline(socknum,result,9999);

		if ( *result!='+' || err < 0) {
			error(FLAG_RECV, "LIST", result);
			close(socknum);
			return ERR_RETR;
		}
		begin=(strstr(result," ")+1);
		begin=(strstr(begin," ")+1);

		unsigned long size=0;

		size=atol(begin);

		snprintf(lbl,100,_("Receiving Message %i of %li (%lu bytes)..."), i, numMsg, size);
		UpdateLabel(lbl);

						

		char *msg=new char[size]; size=0;
		char *ptr=msg;
		char *UID;
		
		if (UIDL) {
			memcpy(command,"UIDL",4);
			send(socknum,command,strlen(command),0);
			err=recvline(socknum,result,9999);

			if (*result!='+' || err < 0) {
				error(FLAG_RECV, "UIDL", result);
				close(socknum);
				return ERR_RETR;
			}
			UID=(strstr(result," ")+1);
			UID=(strstr(UID," ")+1);

			if (seek_in_file(uidfile,UID)) continue;

			FILE *uids=fopen(uidfile,"a+");
			fwrite(UID,strlen(UID),1,uids);
			fwrite("\n",1,1,uids);
			fclose(uids);
			
		}

		memcpy(command,"RETR",4);
		send(socknum,command,strlen(command),0);
		err=recvline(socknum,result,9999);
		if ( *result!='+' || err < 0) {
			error(FLAG_RECV, "RETR",result);
			close(socknum);
			return ERR_RETR;
		}
		
		char *line=new char[10000]; int length;
		do {
			length=recvline(socknum,line,9999);
			for (int z=0; z<length; z++) ptr[z]=line[z];
			ptr+=length; size+=length;
		}
		while (strcmp(line,".\n")!=0);

		if (DELE) { 
			memcpy(command,"DELE",4);
			send(socknum,command,strlen(command),0);
			err=recvline(socknum,result,9999);
			if (*result!='+' || err <0 ) {
				error(FLAG_RECV, "DELE", result);
				close(socknum);
				return ERR_RETR;
			}
		}

		msg[size-2] = 0;
		gchar* msg_p; // msg_prepared - for writing
		msg_p = new gchar[10 + size];
		sprintf(msg_p, "X-MMC: 0\n%s", msg);
		folder->addMail(msg_p);
		delete[] msg_p;
/*		fwrite("From someone Sun 1 Jan 12:00:00 1980\n",37,1,inbox);
		fwrite("X-MMC: 0\n",9,1,inbox);
		fwrite(msg,size-2,1,inbox);
		fwrite("\n",1,1,inbox);
*/		
		UpdateProgress(atol(msg_num), numMsg);
	}

	send(socknum,"QUIT\r\n",6,0);
	err=recvline(socknum,result,9999);

	if(*result!='+' || err < 0) {
		error(FLAG_RECV, "QUIT",result);
		close(socknum);
		return ERR_OTHER;
	}

	close(socknum);
	
	EndProgress();
	
	char msg[101];
	snprintf(msg,100,_("Retrieved %li messages."), numMsg);
	gnome_app_message(GNOME_APP(app), msg);

	folder->parse(); // TODO let user configure if he wants folder to be displayed after
	                 // fetching  mails

	return	OK;
}

CProtoPop3::CProtoPop3() {
}

GtkWidget* CProtoPop3::config(PROTO_FLAGS flag, const gchar* account, const gchar *string,
	GtkWidget *prefs) {
	if (flag != FLAG_RECV) return NULL;

	GtkWidget *theConfig;

	gchar *server, *user, *pass, *port;
	server = gnome_config_get_string("/mmc/pop3/server=");
	user = gnome_config_get_string("/mmc/pop3/user=");
	pass = gnome_config_private_get_string("/mmc/pop3/pass=");
	port = gnome_config_get_string("/mmc/pop3/port=110");

	conf_data.server = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(conf_data.server), server);

	conf_data.port = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(conf_data.port), port);
	
	conf_data.user = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(conf_data.user), user);
	
	conf_data.pass = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(conf_data.pass), pass);

	g_free(server);
	g_free(user);
	g_free(pass);
	g_free(port);

	CAN_CHANGE(conf_data.server)
	CAN_CHANGE(conf_data.port)
	CAN_CHANGE(conf_data.user)
	CAN_CHANGE(conf_data.pass)

	theConfig = gtk_table_new(2, 4, FALSE);

	gtk_table_attach(GTK_TABLE(theConfig), gtk_label_new(_("Server")),
		0, 1, 0, 1, GTK_EXPAND, GTK_EXPAND, 5, 5);
	gtk_table_attach(GTK_TABLE(theConfig), conf_data.server,
		1, 2, 0, 1, GTK_EXPAND, GTK_EXPAND, 5, 5);

	gtk_table_attach(GTK_TABLE(theConfig), gtk_label_new(_("Port")),
		2, 3, 0, 1, GTK_EXPAND, GTK_EXPAND, 5, 5);
	gtk_table_attach(GTK_TABLE(theConfig), conf_data.port,
		3, 4, 0, 1, GTK_EXPAND, GTK_EXPAND, 5, 5);

	gtk_table_attach(GTK_TABLE(theConfig), gtk_label_new(_("User")),
		0, 1, 1, 2, GTK_EXPAND, GTK_EXPAND, 5, 5);
	gtk_table_attach(GTK_TABLE(theConfig), conf_data.user,
		1, 2, 1, 2, GTK_EXPAND, GTK_EXPAND, 5, 5);

	gtk_table_attach(GTK_TABLE(theConfig), gtk_label_new(_("Password")),
		2, 3, 1, 2, GTK_EXPAND, GTK_EXPAND, 5, 5);
	gtk_table_attach(GTK_TABLE(theConfig), conf_data.pass,
		3, 4, 1, 2, GTK_EXPAND, GTK_EXPAND, 5, 5);

	gtk_entry_set_visibility(GTK_ENTRY(conf_data.pass), FALSE);

	return theConfig;	

}

gchar *CProtoPop3::applyConfig(PROTO_FLAGS flag, GtkWidget *widget) {
	gnome_config_set_string("/mmc/pop3/server",
		gtk_entry_get_text(GTK_ENTRY(conf_data.server)));
	gnome_config_set_string("/mmc/pop3/user",
		gtk_entry_get_text(GTK_ENTRY(conf_data.user)));
	gnome_config_private_set_string("/mmc/pop3/pass",
		gtk_entry_get_text(GTK_ENTRY(conf_data.pass)));
	gnome_config_set_string("/mmc/pop3/port",
		gtk_entry_get_text(GTK_ENTRY(conf_data.port)));

	return "def_acc";
}

