/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/
/* Note: The functions recvline and recvch are (c) 1999 Susixware
 * I took them from the source of their Program Spruce, version 0.5.15.
 * Spruce available at: http://www.xtorshun.org/spruce/ or
 * http://spruce.sourceforge.net
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <gtk/gtk.h>
#include <string.h>
#include <string>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <errno.h>
#include <netdb.h>
#include "msg.h"
#include "main.h"
#include "mime.h"
#include "protocols.h"
#include "defaults.h"
#include "progress.h"
#include "pop3.h"

// TODO: make the following List User-Configurable
gchar const * headers[] = {
				"From:",
				"Date:",
				"Subject:",
				"Reply-To:",
				"To:",
				"CC:",
				"X-Mailer:",
				NULL
};

ssize_t recvch(int sock, char *ptr) // (c) 1999 Susixware
{
   static int read_cnt = 0;
   static char *read_ptr;
   static char read_buf[512];

   if (read_cnt <= 0)
   {
      again:
         if ( (read_cnt = recv(sock, read_buf, sizeof(read_buf),0)) < 0)
         {
            if (errno == EINTR)
               goto again;
            return -1;
         }
         else if (read_cnt == 0)
            return 0;
         read_ptr = read_buf;
   }
   read_cnt--;
   *ptr = *read_ptr++;
   return 1;
}

void append(gpointer data1, gpointer data2) {
	CQueryable* fold = (CQueryable*)data1;
	GtkWidget *list = (GtkWidget*)data2;
	
	gchar *id = fold->query();
	
	gtk_clist_append(GTK_CLIST(list), &id);
}

// appends the query string to a glist
void appendgl(gpointer data1, gpointer data2) {
	CQueryable *prot = (CQueryable*)data1;

	listflag *lf = (listflag*)data2;

	if (!(prot->flags() & lf->flags)) return;
	gchar *id = prot->query();

	lf->list = g_list_prepend(lf->list, id);
}


void CFolders::add(CFolder* folder, gchar* name) {
	folder_info* fi = new folder_info;
	fi->name = g_strdup(name);
	fi->folder = folder;
	folders = g_list_append(folders, fi);
}


folder_info* CFolders::getfi(gchar* name) {
	folder_info* fi;
	// GList* folders
	for (guint i = 0; i < g_list_length(folders); i++) {
		fi = (folder_info*)g_list_nth_data(folders, i);
		if (strcasecmp(fi->name, name) == 0)
			return fi;
	}
	return NULL;
}

void CFolders::enumerate(gboolean (callback)(CFolder*,gchar*)) {
	folder_info* fi;
	// GList* folders
	for (guint i = 0; i < g_list_length(folders); i++) {
		fi = (folder_info*)g_list_nth_data(folders, i);
		if (callback(fi->folder, fi->name) == FALSE)
			return;
	}

}

GtkWidget* CFolders::makeCombo() {
	return NULL; // until this function does something...
}

GtkWidget* CFolders::makeCList() {
	GtkWidget* theList;
	gchar* strings[2];
	strings[1]=NULL;

	theList = gtk_clist_new(1);
	gtk_clist_set_selection_mode(GTK_CLIST(theList), GTK_SELECTION_SINGLE);


	folder_info* fi;
	// GList* folders
	for (guint i = 0; i < g_list_length(folders); i++) {
		fi = (folder_info*)g_list_nth_data(folders, i);
		strings[0] = fi->name;
		gint count = gtk_clist_append(GTK_CLIST(theList), strings);
		gtk_clist_set_row_data(GTK_CLIST(theList), count, fi->folder);
	}

	gtk_clist_set_column_width(GTK_CLIST(theList), 0, 
		gtk_clist_optimal_column_width(GTK_CLIST(theList), 0));

	return theList;

}


int CProto::conn(const char *hostname, int port) {
	int socknum;
	struct hostent *hostInfo;
	struct sockaddr_in sin;

	if (isdigit(*hostname))
		sin.sin_addr.s_addr=inet_addr(hostname);
	else if ((hostInfo = gethostbyname(hostname)))
		memcpy((char*)&sin.sin_addr, hostInfo->h_addr, hostInfo->h_length);
	else
		return 0;

	bzero(&(sin.sin_zero), 8);
	sin.sin_port=htons(port);
	sin.sin_family=AF_INET;
  
	socknum = socket(AF_INET, SOCK_STREAM, 0);
 
	if (socknum < 0) return 0;
	if (connect(socknum, (struct sockaddr*)&sin, sizeof(sin)) < 0) return 0;
	return socknum;
}

int CProto::conn(guint32 ip, int port) {
	int socknum, ret;
	struct sockaddr_in sin;

	sin.sin_addr.s_addr=htonl(ip);
	bzero(&(sin.sin_zero), 8);
	sin.sin_port=htons(port);
	sin.sin_family=AF_INET;

	socknum=socket(AF_INET, SOCK_STREAM, 0);

	if (socknum<0) return 0;

	ret=connect(socknum, (struct sockaddr*)&sin, sizeof(sin));
	if (ret<0) return 0;

	return socknum;
}

ssize_t CProto::recvline(int sock, char *vptr, size_t maxlen) // (c) 1999 Susixware
{
   unsigned int n;
   int rc;
   char c, *ptr;

   ptr = vptr;
   for (n = 1; n < maxlen; n++)
   {
      if ( (rc = recvch(sock, &c)) == 1)
      {
         if (c=='\r') { n--; continue; }
         *ptr++ = c;
         if (c == '\n') 
         break;                         /* newline is stored, like fgets() */
      }
      else if (rc == 0)
      {
         if (n == 1)
         {
            *ptr = '\0';
            return 0;                   /* EOF, no data read */
         }
         else
            break;                      /* EOF, some data read */
      }
      else
         return -1;                     /* error, errno set by read() */
   }
	
   *ptr = 0;                            /* null terminate like fgets() */
   return n;
}

int CFolder::conn(const char *hostname, int port) {
	int socknum;
	struct hostent *hostInfo;
	struct sockaddr_in sin;

	if (isdigit(*hostname))
		sin.sin_addr.s_addr=inet_addr(hostname);
	else if ((hostInfo = gethostbyname(hostname)))
		memcpy((char*)&sin.sin_addr, hostInfo->h_addr, hostInfo->h_length);
	else
		return 0;

	bzero(&(sin.sin_zero), 8);
	sin.sin_port=htons(port);
	sin.sin_family=AF_INET;
  
	socknum = socket(AF_INET, SOCK_STREAM, 0);
 
	if (socknum < 0) return 0;
	if (connect(socknum, (struct sockaddr*)&sin, sizeof(sin)) < 0) return 0;
	return socknum;
}

int CFolder::conn(guint32 ip, int port) {
	int socknum, ret;
	struct sockaddr_in sin;

	sin.sin_addr.s_addr=htonl(ip);
	bzero(&(sin.sin_zero), 8);
	sin.sin_port=htons(port);
	sin.sin_family=AF_INET;

	socknum=socket(AF_INET, SOCK_STREAM, 0);

	if (socknum<0) return 0;

	ret=connect(socknum, (struct sockaddr*)&sin, sizeof(sin));
	if (ret<0) return 0;

	return socknum;
}

ssize_t CFolder::recvline(int sock, char *vptr, size_t maxlen) // (c) 1999 Susixware
{
   unsigned int n;
   int rc;
   char c, *ptr;

   ptr = vptr;
   for (n = 1; n < maxlen; n++)
   {
      if ( (rc = recvch(sock, &c)) == 1)
      {
         if (c=='\r') { n--; continue; }
         *ptr++ = c;
         if (c == '\n') 
         break;                         /* newline is stored, like fgets() */
      }
      else if (rc == 0)
      {
         if (n == 1)
         {
            *ptr = '\0';
            return 0;                   /* EOF, no data read */
         }
         else
            break;                      /* EOF, some data read */
      }
      else
         return -1;                     /* error, errno set by read() */
   }
	
   *ptr = 0;                            /* null terminate like fgets() */
   return n;
}

void CProto::error(PROTO_FLAGS flag, const gchar *where, const gchar *line) {
	gchar msg[401];
	gchar tmp[101];

	EndProgress();

	if (line) snprintf(tmp,100,_("Server said: %s\n"),line);
		else *tmp=0;
	
	snprintf(msg,400,_("Error %s mails.\n%s failed.\n%s"),
			flag == FLAG_SEND ? "sending" : "retrieving",
			where,tmp);
	gnome_app_error(GNOME_APP(app), msg);
}

int CFolder::addMail(const gchar* from, const gchar *subject, const gchar *date,
			void *data) {
	gchar *cols[3];
	cols[0]=g_strdup(subject);
	cols[1]=g_strdup(date);
	cols[2]=g_strdup(from);
	gint row=gtk_clist_append(mail_list, cols);
	for (int i=0; i<3; i++) g_free(cols[i]);
	
	CMessage *msg = new CMessage(this, data);
	gtk_clist_set_row_data(mail_list, row, msg);
	return row;
}
gchar *dup(const gchar *string) {
	gchar *res = new gchar[strlen(string)+1];
	strcpy(res,string);
	return res;
}
gchar *oneLine(const gchar *lines, const gchar *sep) {
	if (!lines) return NULL;
	if (*lines == *sep) return dup(sep);
	gchar *end = strstr(lines,sep);
	gint len;
	gchar *line;
	if (!end) len = strlen(lines);
	else len = end - lines;
	line = new gchar[len+2];
	strncpy(line,lines,len);
	line[len]=*sep;
	line[len+1]=0;
	
	return line;
}

// Gets the header specified in the second argument
gchar *getHead(const gchar *message, const gchar *header) {
	gchar *head;
	do {
		if (strncmp(message,"\n\n",2)==0) return NULL;
		message++;
		head = oneLine(message);
		if (strncasecmp(head,header,strlen(header)) == 0) return head;
		else delete[] head;
	} while ((message=strstr(message,"\n")));
	return NULL;
}

// THe following is an awfully complex function
// which is very slow and could be optimized
// actually, I even got an idea on how to make this better:
// Check for the Headers using getHead
int CFolder::select(const gchar *message, gboolean all_headers, gboolean isqp, gint row, void* data) {
	if (!message) return -1;
	const gchar* begin = message;
	gchar *line;
	gchar boundary[91];
	gchar boundend[93];
	gchar *fname=NULL;
	boundary[0]=0; boundend[0]=0;
	gboolean header=TRUE;
	gboolean IsMime=FALSE;
	GdkColor head_color, quote_color, COL_ATTACH;
	GdkFont *mail_font;
	gboolean InBound=FALSE, isText=TRUE, mailtext=TRUE;
	gchar *ttype=dup("text/plain");
	GdkColor col;

	gint len = strlen(message);
	CONTENT_TYPE type=TEXT_PLAIN;
	TRANSFER_ENCODING encoding=BIT8;

	gtk_clist_set_foreground(mail_list,row,NULL);

	head_color.red=gnome_config_get_int("/mmc/colors/head.red=" DEFAULT_HEAD_RED );
	head_color.green=gnome_config_get_int("/mmc/colors/head.green=" DEFAULT_HEAD_GREEN );
	head_color.blue=gnome_config_get_int("/mmc/colors/head.blue=" DEFAULT_HEAD_BLUE );

	quote_color.red=gnome_config_get_int("/mmc/colors/quote.red=" DEFAULT_QUOTE_RED);
	quote_color.green=gnome_config_get_int("/mmc/colors/quote.green=" DEFAULT_QUOTE_GREEN);
	quote_color.blue=gnome_config_get_int("/mmc/colors/quote.blue=" DEFAULT_QUOTE_BLUE);
	
	COL_ATTACH.red=gnome_config_get_int("/mmc/colors/attach.red=" DEFAULT_ATTACH_RED);
	COL_ATTACH.green=gnome_config_get_int("/mmc/colors/attach.green=" DEFAULT_ATTACH_GREEN);
	COL_ATTACH.blue=gnome_config_get_int("/mmc/colors/attach.blue=" DEFAULT_ATTACH_BLUE);


	gchar *font_name=gnome_config_get_string("/mmc/fonts/mail=" DEFAULT_MAIL_FONT);
	mail_font=gdk_font_load(font_name);
	g_free(font_name);

	gtk_text_freeze(text);
	gtk_text_set_point(text,0);
	gtk_editable_delete_text(GTK_EDITABLE(text),0,-1);
	for (;;) {
		line = oneLine(message);
		message += strlen(line);
		
		if ((message-begin)>len) break;
		gchar *up=ucase(line);

		if (*line=='>') col = quote_color;
		else { col.red=0; col.green=0; col.blue=0; }

		if (all_headers && header) gtk_text_insert(text,mail_font,&head_color,NULL,line,-1);
		else if (header) for (const gchar **x=headers; *x; x++) {
			if (strncasecmp(line,*x,strlen(*x))==0) {
				char * up=ucase(line);
				decode_subj(line);
				delete[] up;
				gtk_text_insert(text,mail_font,&head_color,NULL,line,-1);
				
				delete[] line;
				line = oneLine(message);
				if (*line==' ' || *line=='\t') do {
					decode_subj(line);
					gtk_text_insert(text,mail_font,&head_color,NULL,line,-1);
					delete[] line;
					line = oneLine(message);
				} while (*line == ' ' || *line=='\t');
				*line=0;
			}
		}	
		else if (memcmp(line,"From ",5)==0) break;

		if (header && memcmp(up,"MIME-VERSION: ",14)==0) {
			if (memcmp(line+14,"1.0",3)!=0)
				g_print(_("Msg #%i is MIME Message with a version number we don't support! Trying to parse anyway...\n"),
					row+1);
			IsMime=TRUE;
		}
		else if (/*IsMime &&*/ header && memcmp(up,"CONTENT-TYPE: ",14)==0) {
			type=gettype(up);
			delete[] ttype;
			ttype=dup(line+14);
			gchar *x;
			if ((x=(strstr(ttype,";")))) *x=0;
			x=ttype;
			while (*x == ' ') x++;
			x=dup(x);
			delete[] ttype;
			ttype=dup(x);
			delete[] x;

			if (strstr(up+14,"MULTIPART/")) {
				// Find out what the boundary is:
				if (!strstr(up,"BOUNDARY")) {
					delete[] line;
					line=oneLine(message);
					message+=strlen(line);
				}
				if (strstr(line,"boundary")) {
					gchar *myptr=dup(line);
					gchar *pos=strstr(myptr,"=")+1;
					while (*pos==' ') pos++;
					if (*pos=='\"') pos++;
	
					if ((x=(strstr(pos,"\"")))) *x=0;
					if ((x=(strstr(pos,"\n")))) *x=0;

					snprintf(boundary,90,"--%s",pos);
					snprintf(boundend,92,"%s--",boundary);
					delete[] myptr;
				}
			}
			else if (strncasecmp(ttype,"TEXT/", 5)) isText=FALSE;

		}
		else if (IsMime && header && memcmp(up,"CONTENT-TRANSFER-ENCODING: ",27)==0) {
			encoding=getencoding(up);
		}
		else if (header && memcmp(line,"X-MMC:",6)==0) {
			markRead(data);

			// Change statusbar text
			gchar lbl[150];
			guint rows=GTK_CLIST(mail_list)->rows;
			gint unread=GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(mail_list), "unread"));
			snprintf(lbl,150,_("%i Messages in the folder (%i unread)"), rows, (unread>0)?--unread:unread);
			gtk_object_set_data(GTK_OBJECT(mail_list), "unread", GINT_TO_POINTER(unread));
			gnome_appbar_pop(GNOME_APPBAR(status));
			gnome_appbar_push(GNOME_APPBAR(status),lbl);
		}
		else if (*boundend && !header && IsMime && memcmp(line,boundend,strlen(boundend))==0) break;
		else if (*boundary && !header && IsMime && memcmp(line,boundary,strlen(boundary))==0) {
			// Got Message Boundary
			fname=NULL;
			if (InBound) gtk_text_insert(text,NULL,&COL_ATTACH,NULL,
					"---------------------------------------------------\n",-1);
			InBound=TRUE;
			while (*line!='\n') {
				line = oneLine(message);
				message += strlen(line);
				delete[] up;
				up=ucase(line);
				if (memcmp(up,"CONTENT-TRANSFER-ENCODING: ",27)==0) {
				       encoding=getencoding(up);
				}	       
				else if (memcmp(up,"CONTENT-TYPE: ",14)==0) {
					type=gettype(up);
					delete[] ttype;
					ttype=dup(line+14);
					char *x;
					if ((x=(strstr(ttype,";")))) *x=0;
					x=ttype;
					while (*x == ' ') x++;
					x=dup(x);
					delete[] ttype;
					ttype=x;
					if (strncasecmp(ttype,"text/",5)==0) isText=TRUE;
					else isText=FALSE;
			
				}
				else if (memcmp(up,"CONTENT-DISPOSITION: ",21)==0) {
					if (!strstr(up,"FILENAME=")) {
						line = oneLine(message);
						message += strlen(line);
					}
					if (strstr(line,"filename=")) {
						fname = decode_subj(g_strdup( strstr(line,"filename=")+9 ));
					}
					*line = 0;
				}
			}
			attach.add(this, data, (message - begin), (mailtext && text)?_("Mailtext"):fname,
				ttype, encoding, boundary);
			mailtext = FALSE;
			if (!isText) {
				char string[300];
				snprintf(string,299,_("Attachment: Has undisplayable Type: %s\nFilename: %s\n"),ttype, 
					fname!=NULL?fname:_("unknown"));
				gtk_text_insert(text,NULL,&COL_ATTACH,NULL,string,-1);
				if (fname) g_free(fname);
			}

		}
		else if (header && *line=='\n') {
			header=FALSE;
			gtk_text_insert(text,NULL,NULL,NULL,"\n",-1);
			if (!isText) gtk_text_insert(text,NULL,&COL_ATTACH,NULL,
				_("Message is not Text - can't be displayed!"), -1);
		}
		else if (!header && ((IsMime && encoding==QUOTED_PRINTABLE && (strncasecmp(ttype,"TEXT/",5)==0)) || isqp)) {
			decode_mime(line);
			gtk_text_insert(text,mail_font,&col,NULL,line,-1);
		}
		else if (!header && ((IsMime && encoding==BASE64 && (strncasecmp(ttype,"TEXT/",5)==0)) || isqp)) {
			guint len;
			string str(line);
			for (;;) {
				delete[] line;
				line = oneLine(message);
				if (!line) break;
				message += strlen(line);
				
				if (
					(memcmp(line,"From ",5)==0) ||
					(memcmp(line,boundary,strlen(boundary))==0)
				)
					break;
				else str.append(line);
			}
			message -= strlen(line);
			(*line)=0;
			gchar *dLine = decode_b64(str.c_str(),&len);
			gtk_text_insert(text,mail_font,&col,NULL,dLine,len);
			delete[] dLine;
		}
		else if (!header && (!IsMime || (strncasecmp(ttype,"TEXT/",5)==0)))
			gtk_text_insert(text,mail_font,&col,NULL,line,-1);
		delete[] line;
		delete[] up;
	}
	gtk_text_thaw(text);
	delete[] ttype;
	delete[] line;

	return 0;

}
