/*
       MMC (My Mail Client). The name says it all. A Gnome Mail Client.
       Copyright (C) 2000  Christian Biesinger <biesi@users.sourceforge.net> and
       Steven Behnke <sbehnke@digistatic.net>

       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
       the Free Software Foundation; either version 2 of the License, or
       (at your option) any later version.

       This program is distributed in the hope that it will be useful,
       but WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
       GNU General Public License for more details.

       You should have received a copy of the GNU General Public License
       along with this program; if not, write to the Free Software
       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

       For the complete License see the file COPYING.
*/

#ifdef HAVE_CONFIG_H
# include<config.h>
#endif
#include<sys/socket.h>
#include<gnome.h>

//#include"pop3.h"
#include"smtp.h"
#include"main.h"

int send_mail(GtkWidget *widget, const gchar *from, gchar *to[], const gchar* data) {
	gchar *protname, *idstring;
	gint len;
	gchar *accountname = gnome_config_get_string("/mmc/accounts/current");

	if (*accountname == 0) {
		gnome_app_error(GNOME_APP(app),
			_("Please configure MMC before attempting to send mails!"));
	}
	
	gchar *qry_string = new gchar[31];
	snprintf(qry_string,30,"/mmc/accounts/%s",accountname);

	gchar *cnf_string = gnome_config_get_string(qry_string);

	gchar *ptr = strstr(cnf_string,"@@");
	ptr = strstr((ptr+2), "@@");
	g_assert(ptr != NULL);
	ptr += 2; // ptr is begin of type of Send "Plugin"

	gchar *ptr2 = strstr(ptr, "@@");


	len = ptr2-ptr;

	protname = new gchar[len+1];
	protname[len]=0;
	if (ptr2) strncpy(protname, ptr, len);
	else {
		gnome_app_error(GNOME_APP(app),
		_("Could not parse Account Configuration String"
		"Maybe you have not configured MMC?"));
		return -1;
	}
	#ifdef DEBUG
	g_print("Send Protocol Type: %s\n", protname);
	#endif

	ptr2 += 2;
	len = strlen(ptr2);

	idstring = new gchar[len+1];
	idstring[len]=0;
	strncpy(idstring, ptr2, len);	

	#ifdef DEBUG
	g_print("Send IDSTRING: %s\n", idstring);
	#endif

	CProto *protocol = g_pHandler.get(protname);
	g_assert(protocol != NULL);

	gint result = protocol->send(idstring, from, to, data, widget);

	g_free(accountname);
	g_free(qry_string);
	g_free(cnf_string);
	delete[] protname;
	delete[] idstring;

	return result;
	
}

void smtp_error(const char *where, const char *line, GtkWidget *widget) {
	char msg[501];
	snprintf(msg,500,_("Error sending message (%s failed).\nServer said %s"),where,line);
	if (widget && GNOME_IS_APP(widget)) gnome_app_error(GNOME_APP(widget), msg);
	else if (widget && GTK_IS_WINDOW(widget)) gnome_error_dialog_parented(msg,GTK_WINDOW(widget));
	else g_warning(msg);
}

GtkWidget* CProtoSmtp::config(PROTO_FLAGS flag, const gchar* account, const gchar
	*yourstring, GtkWidget *prefs) {

	GtkTable *smtp_cnf = GTK_TABLE(gtk_table_new(1, 2, FALSE));
	GtkWidget *w = gnome_entry_new("smtp_serv");
	GtkWidget *gtkw = gnome_entry_gtk_entry(GNOME_ENTRY(w));
	
	gchar *srv = gnome_config_get_string("/mmc/smtp/server=localhost");
	gtk_entry_set_text(GTK_ENTRY(gtkw), srv);
	g_free(srv);

	CAN_CHANGE(gtkw)

	cnf_smtp_serv = GTK_ENTRY(gtkw);

	gtk_table_attach_defaults(smtp_cnf, gtk_label_new(_("SMTP Server")), 0, 1, 0, 1);
	gtk_table_attach_defaults(smtp_cnf, w, 1, 2, 0, 1);

	return GTK_WIDGET(smtp_cnf);
}
 

gint CProtoSmtp::send(const gchar* serv, const gchar* from, gchar* rcpt[], const gchar* data, GtkWidget* widget) {
	gchar line[201];
	gint socknum=conn(serv,25);
	gint err;

	if (socknum == 0) {
		smtp_error(_("Connect"), "", widget);
		return -1;
	}

	gchar command[200];
	err=recvline(socknum,line,200);
	
	snprintf(command,199,"HELO %s\r\n",(strstr(from,"@")+1));
	send(socknum,command,strlen(command),0);

	err=recvline(socknum,line,200);

	while (line[3]=='-') {
		err=recvline(socknum,line,200);
		if (err<0 || memcmp(line,"250",3)) {
			close(socknum);
			smtp_error("HELO",line,widget);
			return -1;
		}
	}

	snprintf(command,199,"MAIL FROM:<%s>\r\n",from);
	send(socknum,command,strlen(command),0);
	err=recvline(socknum,line,200);
	if (err<0 || memcmp(line,"250",3)) { close(socknum); smtp_error("MAIL FROM",line,widget); return -1; }
	
	for (gchar **to=rcpt; *to!=NULL; to++) {
		snprintf(command,sizeof(command)-1,"RCPT TO:<%s>\r\n",*to);
		send(socknum,command,strlen(command),0);
		err=recvline(socknum,line,200);
		if (err<0 || memcmp(line,"250",3)) { close(socknum); smtp_error("RCPT TO",line,widget); return -1; }
	}
	snprintf(command,sizeof(command)-1,"DATA\r\n");
	send(socknum,command,strlen(command),0);
	err=recvline(socknum,line,200);

	if (err<0 || memcmp(line,"354",3)) { close(socknum); smtp_error("DATA",line,widget); return -1; }

	send(socknum,data,strlen(data),0);
	send(socknum,"\r\n.\r\n",5,0);

	err=recvline(socknum,line,200);
	if (err<0 || memcmp(line,"250",3)) { close(socknum); smtp_error("DATA",line,widget); return -1; }

	snprintf(command,sizeof(command)-1,"QUIT\r\n");
	send(socknum,command,strlen(command),0);
	err=recvline(socknum,line,100);

	if (err<0 || memcmp(line,"221",3)) { close(socknum); smtp_error("QUIT",line,widget); return -1; }
	close(socknum);

	if (widget) {
		gtk_widget_destroy(widget);
		gnome_app_message(GNOME_APP(app),_("Message sent successfully."));
	}

	return 0;
}
